@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008 Olly Betts
  * Copyright 2007 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -44,6 +44,7 @@
 #include <vector>
 #include "autoptr.h"
 #include <algorithm>
+#include <cfloat>
 #include <math.h>
 
 using namespace std;
@@ -107,15 +108,13 @@ RSet::contains(Xapian::docid did) const
 string
 RSet::get_description() const
 {
-    DEBUGCALL(INTRO, string, "RSet::get_description", "");
-    RETURN("RSet(" + internal->get_description() + ")");
+    return "RSet(" + internal->get_description() + ")";
 }
 
 string
 RSet::Internal::get_description() const
 {
-    DEBUGCALL(INTRO, string, "RSet::get_description", "");
-    string description;
+    string description("RSet::Internal(");
 
     set<Xapian::docid>::const_iterator i;
     for (i = items.begin(); i != items.end(); ++i) {
@@ -125,7 +124,7 @@ RSet::Internal::get_description() const
 
     description = "RSet(" + description + ")";
 
-    RETURN(description);
+    return description;
 }
 
 namespace Internal {
@@ -135,7 +134,6 @@ namespace Internal {
 string
 MSetItem::get_description() const
 {
-    DEBUGCALL(INTRO, string, "Xapian::MSetItem::get_description", "");
     string description;
 
     description = om_tostring(did) + ", " + om_tostring(wt) + ", " +
@@ -143,7 +141,7 @@ MSetItem::get_description() const
 
     description = "Xapian::MSetItem(" + description + ")";
 
-    RETURN(description);
+    return description;
 }
 
 }
@@ -336,19 +334,19 @@ MSet::back() const
 string
 MSet::get_description() const
 {
-    DEBUGCALL(INTRO, string, "Xapian::MSet::get_description", "");
     Assert(internal.get() != 0);
-    RETURN("Xapian::MSet(" + internal->get_description() + ")");
+    return "Xapian::MSet(" + internal->get_description() + ")";
 }
 
 percent
 MSet::Internal::convert_to_percent_internal(Xapian::weight wt) const
 {
-    DEBUGAPICALL(Xapian::percent, "Xapian::MSet::Internal::convert_to_percent", wt);
+    DEBUGAPICALL(Xapian::percent, "Xapian::MSet::Internal::convert_to_percent_internal", wt);
     if (percent_factor == 0) RETURN(100);
 
-    // FIXME: + 0.5 ???
-    Xapian::percent pcent = static_cast<Xapian::percent>(wt * percent_factor);
+    // Excess precision on x86 can result in a difference here.
+    double v = wt * percent_factor + 100.0 * DBL_EPSILON;
+    Xapian::percent pcent = static_cast<Xapian::percent>(v);
     DEBUGLINE(API, "wt = " << wt << ", max_possible = "
 	      << max_possible << " =>  pcent = " << pcent);
     if (pcent > 100) pcent = 100;
@@ -444,8 +442,7 @@ MSet::Internal::read_docs() const
 string
 Xapian::Internal::ESetItem::get_description() const
 {
-    DEBUGCALL(INTRO, string, "Xapian::Internal::ESetItem::get_description", "");
-    RETURN("Xapian::Internal::ESetItem(" + tname + ", " + om_tostring(wt) + ")");
+    return "Xapian::Internal::ESetItem(" + tname + ", " + om_tostring(wt) + ")";
 }
 
 // Methods for Xapian::ESet
@@ -522,9 +519,8 @@ ESet::back() const
 string
 ESet::get_description() const
 {
-    DEBUGCALL(INTRO, string, "Xapian::ESet::get_description", "");
     Assert(internal.get() != 0);
-    RETURN("Xapian::ESet(" + internal->get_description() + ")");
+    return "Xapian::ESet(" + internal->get_description() + ")";
 }
 
 //////////////////////////////////
@@ -534,7 +530,6 @@ ESet::get_description() const
 string
 Xapian::ESet::Internal::get_description() const
 {
-    DEBUGCALL(INTRO, string, "Xapian::ESet::Internal::get_description", "");
     string description = "ebound=" + om_tostring(ebound);
 
     for (vector<Xapian::Internal::ESetItem>::const_iterator i = items.begin();
@@ -543,7 +538,7 @@ Xapian::ESet::Internal::get_description() const
 	description += ", " + i->get_description();
     }
 
-    RETURN("Xapian::ESet::Internal(" + description + ")");
+    return "Xapian::ESet::Internal(" + description + ")";
 }
 
 // Xapian::ESetIterator
@@ -913,6 +908,7 @@ Enquire::set_sort_by_relevance()
 void
 Enquire::set_sort_by_value(Xapian::valueno sort_key, bool ascending)
 {
+    internal->sorter = NULL;
     internal->sort_key = sort_key;
     internal->sort_by = Internal::VAL;
     internal->sort_value_forward = ascending;
@@ -922,6 +918,7 @@ void
 Enquire::set_sort_by_value_then_relevance(Xapian::valueno sort_key,
 					  bool ascending)
 {
+    internal->sorter = NULL;
     internal->sort_key = sort_key;
     internal->sort_by = Internal::VAL_REL;
     internal->sort_value_forward = ascending;
@@ -931,6 +928,7 @@ void
 Enquire::set_sort_by_relevance_then_value(Xapian::valueno sort_key,
 					  bool ascending)
 {
+    internal->sorter = NULL;
     internal->sort_key = sort_key;
     internal->sort_by = Internal::REL_VAL;
     internal->sort_value_forward = ascending;
@@ -1046,8 +1044,7 @@ Enquire::register_match_decider(const string &name,
 string
 Enquire::get_description() const
 {
-    DEBUGCALL(INTRO, string, "Xapian::Enquire::get_description", "");
-    RETURN("Xapian::Enquire(" + internal->get_description() + ")");
+    return "Xapian::Enquire(" + internal->get_description() + ")";
 }
 
 }
