@@ -131,7 +131,6 @@ Xapian::PostingIterator::positionlist_begin() const
 string
 Xapian::PostingIterator::get_description() const
 {
-    DEBUGCALL(INTRO, string, "Xapian::PostingIterator::get_description", "");
     /// \todo display contents of the object
     string desc = "Xapian::PostingIterator([pos=";
     if (internal.get() == 0) {
@@ -140,5 +139,5 @@ Xapian::PostingIterator::get_description() const
 	desc += internal->get_description();
     }
     desc += "])";
-    RETURN(desc);
+    return desc;
 }
