@@ -23,6 +23,8 @@
 
 #include "safeerrno.h"
 
+#include <xapian/error.h>
+
 #include "flint_cursor.h"
 #include "flint_table.h"
 #include "flint_btreeutil.h"
@@ -179,14 +181,55 @@ FlintCursor::find_entry(const string &key)
 	while (Item_(C[0].p, C[0].c).component_of() != 1) {
 	    if (! B->prev(C, 0)) {
 		is_positioned = false;
-		break;
+		throw Xapian::DatabaseCorruptError("find_entry failed to find any entry at all!");
 	    }
 	}
     }
 done:
 
-    if (!is_positioned) throw Xapian::DatabaseCorruptError("find_entry failed to find any entry at all!");
-    get_key(&current_key);
+    if (found)
+	current_key = key;
+    else
+	get_key(&current_key);
+    tag_status = UNREAD;
+
+    DEBUGLINE(DB, "Found entry: key=`" << hex_encode(current_key) << "'");
+    RETURN(found);
+}
+
+bool
+FlintCursor::find_entry_ge(const string &key)
+{
+    DEBUGCALL(DB, bool, "FlintCursor::find_entry_ge", key);
+    Assert(B->level <= level);
+
+    is_after_end = false;
+
+    bool found;
+
+    is_positioned = true;
+    if (key.size() > FLINT_BTREE_MAX_KEY_LEN) {
+	// Can't find key - too long to possibly be present, so find the
+	// truncated form but ignore "found".
+	B->form_key(key.substr(0, FLINT_BTREE_MAX_KEY_LEN));
+	(void)(B->find(C));
+	found = false;
+    } else {
+	B->form_key(key);
+	found = B->find(C);
+    }
+
+    if (found) {
+	current_key = key;
+    } else {
+	if (! B->next(C, 0)) {
+	    is_after_end = true;
+	    is_positioned = false;
+	    RETURN(false);
+	}
+	Assert(Item_(C[0].p, C[0].c).component_of() == 1);
+	get_key(&current_key);
+    }
     tag_status = UNREAD;
 
     DEBUGLINE(DB, "Found entry: key=`" << hex_encode(current_key) << "'");
@@ -225,16 +268,15 @@ FlintCursor::read_tag(bool keep_compressed)
     return (tag_status == COMPRESSED);
 }
 
-void
+bool
 FlintCursor::del()
 {
     Assert(!is_after_end);
 
-    // FIXME: this isn't the most efficient approach, but I struggled to
-    // make the obvious approaches work.
     B->del(current_key);
-    find_entry(current_key);
-    next();
 
-    DEBUGLINE(DB, "Moved to entry: key=`" << hex_encode(current_key) << "'");
+    // The deletion happens in a new (uncommitted) revision of the tree, but
+    // the cursor is running over the previous revision, so it can still see
+    // the deleted key.
+    return next();
 }
