@@ -639,8 +639,8 @@ FlintTable::enter_key(int j, Key_ prevkey, Key_ newkey)
 
     byte b[UCHAR_MAX + 6];
     Item_wr_ item(b);
-    Assert(I2 + i + C2 <= 256);
-    Assert(I2 + i + C2 + 4 <= (int)sizeof(b));
+    Assert(i <= 256 - I2 - C2);
+    Assert(i <= (int)sizeof(b) - I2 - C2 - 4);
     item.set_key_and_block(newkey, i, blocknumber);
 
     // When j > 1 we can make the first key of block p null.  This is probably
@@ -1026,14 +1026,6 @@ FlintTable::add(const string &key, string tag, bool already_compressed)
 
     if (handle == -1) create_and_open(block_size);
 
-    if (key.size() > FLINT_BTREE_MAX_KEY_LEN) {
-	throw Xapian::InvalidArgumentError(
-		"Key too long: length was " +
-		om_tostring(key.size()) +
-		" bytes, maximum length of a key is " +
-		om_tostring(FLINT_BTREE_MAX_KEY_LEN) + " bytes");
-    }
-
     form_key(key);
 
     bool compressed = false;
@@ -1233,6 +1225,9 @@ FlintTable::find_tag(const string &key, string * tag) const
     DEBUGCALL(DB, bool, "FlintTable::find_tag", key << ", &tag");
     if (handle == -1) RETURN(false);
 
+    // An oversized key can't exist, so attempting to search for it should fail.
+    if (key.size() > FLINT_BTREE_MAX_KEY_LEN) RETURN(false);
+
     form_key(key);
     if (!find(C)) RETURN(false);
 
@@ -1257,7 +1252,9 @@ FlintTable::read_tag(Cursor_ * C_, string *tag, bool keep_compressed) const
     bool compressed = item.get_compressed();
 
     for (int i = 2; i <= n; i++) {
-	next(C_, 0);
+	if (!next(C_, 0)) {
+	    throw Xapian::DatabaseCorruptError("Unexpected end of table when reading continuation of tag");
+	}
 	(void)Item_(C_[0].p, C_[0].c).append_chunk(tag);
     }
     // At this point the cursor is on the last item - calling next will move
@@ -1682,6 +1679,7 @@ FlintTable::create_and_open(unsigned int block_size_)
 
     /* create the base file */
     FlintTable_base base_;
+    base_.set_revision(revision_number);
     base_.set_block_size(block_size_);
     base_.set_have_fakeroot(true);
     base_.set_sequential(true);
@@ -1732,7 +1730,10 @@ FlintTable::commit(flint_revision_number_t revision)
 	throw Xapian::DatabaseError("New revision too low");
     }
 
-    if (handle == -1) return;
+    if (handle == -1) {
+	latest_revision_number = revision_number = revision;
+	return;
+    }
 
     // FIXME: this doesn't work (probably because the table revisions get
     // out of step) but it's wasteful to keep applying changes to value
@@ -1785,7 +1786,33 @@ FlintTable::commit(flint_revision_number_t revision)
 	C[i].rewrite = false;
     }
 
-    base.write_to_file(name + "base" + char(base_letter));
+    // Save to "<table>.tmp" and then rename to "<table>.base<letter>" so that
+    // a reader can't try to read a partially written base file.
+    string tmp = name;
+    tmp += "tmp";
+    string basefile = name;
+    basefile += "base";
+    basefile += char(base_letter);
+    base.write_to_file(tmp);
+#if defined __WIN32__
+    if (msvc_posix_rename(tmp.c_str(), basefile.c_str()) < 0) {
+#else
+    if (rename(tmp.c_str(), basefile.c_str()) < 0) {
+#endif
+	// With NFS, rename() failing may just mean that the server crashed
+	// after successfully renaming, but before reporting this, and then
+	// the retried operation fails.  So we need to check if the source
+	// file still exists, which we do by calling unlink(), since we want
+	// to remove the temporary file anyway.
+	int saved_errno = errno;
+	if (unlink(tmp) == 0 || errno != ENOENT) {
+	    string msg("Couldn't update base file ");
+	    msg += basefile;
+	    msg += ": ";
+	    msg += strerror(saved_errno);
+	    throw Xapian::DatabaseError(msg);
+	}
+    }
     base.commit();
 
     read_root();
@@ -1801,7 +1828,10 @@ FlintTable::cancel()
     DEBUGCALL(DB, void, "FlintTable::cancel", "");
     Assert(writable);
 
-    if (handle == -1) return;
+    if (handle == -1) {
+	latest_revision_number = revision_number; // FIXME: we can end up reusing a revision if we opened a btree at an older revision, start to modify it, then cancel...
+	return;
+    }
 
     // This causes problems: if (!Btree_modified) return;
 
@@ -1931,10 +1961,28 @@ FlintTable::prev_for_sequential(Cursor_ * C_, int /*dummy*/) const
 	while (true) {
 	    if (n == 0) return false;
 	    n--;
-	    // Check if the block is in the built-in cursor (potentially in
-	    // modified form).
-	    if (writable && n == C[0].n) {
-		memcpy(p, C[0].p, block_size);
+	    if (writable) {
+		if (n == C[0].n) {
+		    // Block is a leaf block in the built-in cursor
+		    // (potentially in modified form).
+		    memcpy(p, C[0].p, block_size);
+		} else {
+		    // Blocks in the built-in cursor may not have been written
+		    // to disk yet, so we have to check that the block number
+		    // isn't in the built-in cursor or we'll read an
+		    // uninitialised block (for which GET_LEVEL(p) will
+		    // probably return 0).
+		    int j;
+		    for (j = 1; j <= level; ++j) {
+			if (n == C[j].n) break;
+		    }
+		    if (j <= level) continue;
+
+		    // Block isn't in the built-in cursor, so the form on disk
+		    // is valid, so read it to check if it's the next level 0
+		    // block.
+		    read_block(n, p);
+		}
 	    } else {
 		read_block(n, p);
 	    }
@@ -1959,15 +2007,34 @@ FlintTable::next_for_sequential(Cursor_ * C_, int /*dummy*/) const
     Assert(p);
     int c = C_[0].c;
     c += D2;
+    Assert((unsigned)c < block_size);
     if (c == DIR_END(p)) {
 	uint4 n = C_[0].n;
 	while (true) {
 	    n++;
 	    if (n > base.get_last_block()) return false;
-	    // Check if the block is in the built-in cursor (potentially in
-	    // modified form).
-	    if (writable && n == C[0].n) {
-		memcpy(p, C[0].p, block_size);
+	    if (writable) {
+		if (n == C[0].n) {
+		    // Block is a leaf block in the built-in cursor
+		    // (potentially in modified form).
+		    memcpy(p, C[0].p, block_size);
+		} else {
+		    // Blocks in the built-in cursor may not have been written
+		    // to disk yet, so we have to check that the block number
+		    // isn't in the built-in cursor or we'll read an
+		    // uninitialised block (for which GET_LEVEL(p) will
+		    // probably return 0).
+		    int j;
+		    for (j = 1; j <= level; ++j) {
+			if (n == C[j].n) break;
+		    }
+		    if (j <= level) continue;
+
+		    // Block isn't in the built-in cursor, so the form on disk
+		    // is valid, so read it to check if it's the next level 0
+		    // block.
+		    read_block(n, p);
+		}
 	    } else {
 		read_block(n, p);
 	    }
