@@ -1,7 +1,7 @@
 /* flint_table.h: Btree implementation
  *
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,6 +22,7 @@
 #ifndef OM_HGUARD_FLINT_TABLE_H
 #define OM_HGUARD_FLINT_TABLE_H
 
+#include <xapian/error.h>
 #include <xapian/visibility.h>
 
 #include <algorithm>
@@ -32,6 +33,10 @@ using std::string;
 #include "flint_btreebase.h"
 #include "flint_btreeutil.h"
 #include "flint_cursor.h"
+#include "noreturn.h"
+
+#include "stringutils.h"
+#include "utils.h"
 
 #include <zlib.h>
 
@@ -42,7 +47,7 @@ using std::string;
  *  This gives the upper limit of the size of a key that may be stored in the
  *  B-tree (252 bytes with the present implementation).
  */
-const string::size_type FLINT_BTREE_MAX_KEY_LEN = 252;
+#define FLINT_BTREE_MAX_KEY_LEN 252
 
 // FIXME: This named constant probably isn't used everywhere it should be...
 #define BYTES_PER_BLOCK_NUMBER 4
@@ -196,10 +201,17 @@ public:
 	set_size(I2 + K1 + 4);  /* total length */
     }
     void form_key(const string & key_) {
-	Assert(key_.length() <= FLINT_BTREE_MAX_KEY_LEN);
-
-	// This just so it doesn't fall over horribly in non-debug builds.
-	string::size_type key_len = std::min(key_.length(), FLINT_BTREE_MAX_KEY_LEN);
+	string::size_type key_len = key_.length();
+	if (key_len > FLINT_BTREE_MAX_KEY_LEN) {
+	    // We check term length when a term is added to a document but
+	    // flint doubles zero bytes, so this can still happen for terms
+	    // which contain one or more zero bytes.
+	    string msg("Key too long: length was ");
+	    msg += om_tostring(key_len);
+	    msg += " bytes, maximum length of a key is "
+		   STRINGIZE(FLINT_BTREE_MAX_KEY_LEN) " bytes";
+	    throw Xapian::InvalidArgumentError(msg);
+	}
 
 	set_key_len(key_len + K1 + C2);
 	memmove(p + I2 + K1, key_.data(), key_len);
@@ -555,7 +567,7 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	int delete_kt();
 	void read_block(uint4 n, byte *p) const;
 	void write_block(uint4 n, const byte *p) const;
-	void set_overwritten() const;
+	XAPIAN_NORETURN(void set_overwritten() const);
 	void block_to_cursor(Cursor_ *C_, int j, uint4 n) const;
 	void alter();
 	void compact(byte *p);
