@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -707,8 +707,8 @@ Btree::enter_key(int j, Key prevkey, Key newkey)
 
     byte b[UCHAR_MAX + 6];
     Item_wr item(b);
-    Assert(I2 + i + C2 <= 256);
-    Assert(I2 + i + C2 + 4 <= (int)sizeof(b));
+    Assert(i <= 256 - I2 - C2);
+    Assert(i <= (int)sizeof(b) - I2 - C2 - 4);
     item.set_key_and_block(newkey, i, blocknumber);
 
     // When j > 1 we can make the first key of block p null.  This is probably
@@ -1239,7 +1239,9 @@ Btree::read_tag(Cursor * C_, string *tag) const
     item.append_chunk(tag);
 
     for (int i = 2; i <= n; i++) {
-	next(C_, 0);
+	if (!next(C_, 0)) {
+	    throw Xapian::DatabaseCorruptError("Unexpected end of table when reading continuation of tag");
+	}
 	(void)Item(C_[0].p, C_[0].c).append_chunk(tag);
     }
     // At this point the cursor is on the last item - calling next will move
@@ -1655,6 +1657,33 @@ Btree::commit(quartz_revision_number_t revision)
     }
 
     base.write_to_file(name + "base" + char(base_letter));
+    // Save to "<table>.tmp" and then rename to "<table>.base<letter>" so that
+    // a reader can't try to read a partially written base file.
+    string tmp = name;
+    tmp += "tmp";
+    string basefile = name;
+    basefile += "base";
+    basefile += char(base_letter);
+    base.write_to_file(tmp);
+#if defined __WIN32__
+    if (msvc_posix_rename(tmp.c_str(), basefile.c_str()) < 0) {
+#else
+    if (rename(tmp.c_str(), basefile.c_str()) < 0) {
+#endif
+	// With NFS, rename() failing may just mean that the server crashed
+	// after successfully renaming, but before reporting this, and then
+	// the retried operation fails.  So we need to check if the source
+	// file still exists, which we do by calling unlink(), since we want
+	// to remove the temporary file anyway.
+	int saved_errno = errno;
+	if (unlink(tmp) == 0 || errno != ENOENT) {
+	    string msg("Couldn't update base file ");
+	    msg += basefile;
+	    msg += ": ";
+	    msg += strerror(saved_errno);
+	    throw Xapian::DatabaseError(msg);
+	}
+    }
     base.commit();
 
     read_root();
@@ -1795,10 +1824,28 @@ Btree::prev_for_sequential(Cursor * C_, int /*dummy*/) const
 	while (true) {
 	    if (n == 0) return false;
 	    n--;
-	    // Check if the block is in the built-in cursor (potentially in
-	    // modified form).
-	    if (writable && n == C[0].n) {
-		memcpy(p, C[0].p, block_size);
+	    if (writable) {
+		if (n == C[0].n) {
+		    // Block is a leaf block in the built-in cursor
+		    // (potentially in modified form).
+		    memcpy(p, C[0].p, block_size);
+		} else {
+		    // Blocks in the built-in cursor may not have been written
+		    // to disk yet, so we have to check that the block number
+		    // isn't in the built-in cursor or we'll read an
+		    // uninitialised block (for which GET_LEVEL(p) will
+		    // probably return 0).
+		    int j;
+		    for (j = 1; j <= level; ++j) {
+			if (n == C[j].n) break;
+		    }
+		    if (j <= level) continue;
+
+		    // Block isn't in the built-in cursor, so the form on disk
+		    // is valid, so read it to check if it's the next level 0
+		    // block.
+		    read_block(n, p);
+		}
 	    } else {
 		read_block(n, p);
 	    }
@@ -1828,10 +1875,28 @@ Btree::next_for_sequential(Cursor * C_, int /*dummy*/) const
 	while (true) {
 	    n++;
 	    if (n > base.get_last_block()) return false;
-	    // Check if the block is in the built-in cursor (potentially in
-	    // modified form).
-	    if (writable && n == C[0].n) {
-		memcpy(p, C[0].p, block_size);
+	    if (writable) {
+		if (n == C[0].n) {
+		    // Block is a leaf block in the built-in cursor
+		    // (potentially in modified form).
+		    memcpy(p, C[0].p, block_size);
+		} else {
+		    // Blocks in the built-in cursor may not have been written
+		    // to disk yet, so we have to check that the block number
+		    // isn't in the built-in cursor or we'll read an
+		    // uninitialised block (for which GET_LEVEL(p) will
+		    // probably return 0).
+		    int j;
+		    for (j = 1; j <= level; ++j) {
+			if (n == C[j].n) break;
+		    }
+		    if (j <= level) continue;
+
+		    // Block isn't in the built-in cursor, so the form on disk
+		    // is valid, so read it to check if it's the next level 0
+		    // block.
+		    read_block(n, p);
+		}
 	    } else {
 		read_block(n, p);
 	    }
