@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2006 Olly Betts
+ * Copyright 2002,2003,2004,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,6 +23,8 @@
 #ifndef OM_HGUARD_QUARTZ_DATABASE_H
 #define OM_HGUARD_QUARTZ_DATABASE_H
 
+#include <xapian/visibility.h>
+
 #include "database.h"
 #include "quartz_log.h"
 #include "quartz_metafile.h"
@@ -43,7 +45,7 @@ const int OM_DB_READONLY = 0;
 /** A backend designed for efficient indexing and retrieval, using
  *  compressed posting lists and a btree storage scheme.
  */
-class QuartzDatabase : public Xapian::Database::Internal {
+class XAPIAN_VISIBILITY_DEFAULT QuartzDatabase : public Xapian::Database::Internal {
     friend class QuartzWritableDatabase;
     friend class QuartzTermList;
     private:
@@ -181,7 +183,12 @@ class QuartzDatabase : public Xapian::Database::Internal {
 	 *  @exception Xapian::DatabaseCorruptError is thrown if there is no
 	 *             consistent revision available.
 	 *
-	 *  @exception Xapian::DatabaseOpeningError thrown if database can't be opened.
+	 *  @exception Xapian::DatabaseOpeningError thrown if database can't be
+	 *             opened.
+	 *
+	 *  @exception Xapian::DatabaseVersionError thrown if database is in an
+	 *             unsupported format.  This implies that the database was
+	 *             created by an older or newer version of Xapian.
 	 *
 	 *  @param dbdir directory holding quartz tables
 	 *
@@ -208,18 +215,18 @@ class QuartzDatabase : public Xapian::Database::Internal {
 	bool term_exists(const string & tname) const;
 	bool has_positions() const;
 
-	LeafPostList * do_open_post_list(const string & tname) const;
-	LeafTermList * open_term_list(Xapian::docid did) const;
+	LeafPostList * open_post_list(const string & tname) const;
+	TermList * open_term_list(Xapian::docid did) const;
 	Xapian::Document::Internal * open_document(Xapian::docid did, bool lazy = false) const;
 	PositionList * open_position_list(Xapian::docid did,
 					  const string & tname) const;
-	TermList * open_allterms() const;
+	TermList * open_allterms(const string & prefix) const;
 	//@}
 };
 
 /** A writable quartz database.
  */
-class QuartzWritableDatabase : public Xapian::Database::Internal {
+class XAPIAN_VISIBILITY_DEFAULT QuartzWritableDatabase : public Xapian::Database::Internal {
     private:
 	/** Unflushed changes to term frequencies and collection frequencies. */
 	mutable map<string, pair<Xapian::termcount_diff, Xapian::termcount_diff> >
@@ -263,6 +270,15 @@ class QuartzWritableDatabase : public Xapian::Database::Internal {
 
 	virtual Xapian::docid add_document(const Xapian::Document & document);
 	Xapian::docid add_document_(Xapian::docid did, const Xapian::Document & document);
+	// Stop the default implementation of delete_document(term) and
+	// replace_document(term) from being hidden.  This isn't really
+	// a problem as we only try to call them through the base class
+	// (where they aren't hidden) but some compilers generate a warning
+	// about the hiding.
+#if (!defined __GNUC__ && !defined _MSC_VER) || __GNUC__ > 2
+	using Xapian::Database::Internal::delete_document;
+	using Xapian::Database::Internal::replace_document;
+#endif
 	virtual void delete_document(Xapian::docid did);
 	virtual void replace_document(Xapian::docid did,
 				      const Xapian::Document & document);
@@ -271,7 +287,12 @@ class QuartzWritableDatabase : public Xapian::Database::Internal {
     public:
 	/** Create and open a writable quartz database.
 	 *
-	 *  @exception Xapian::DatabaseOpeningError thrown if database can't be opened.
+	 *  @exception Xapian::DatabaseOpeningError thrown if database can't be
+	 *             opened.
+	 *
+	 *  @exception Xapian::DatabaseVersionError thrown if database is in an
+	 *             unsupported format.  This implies that the database was
+	 *             created by an older or newer version of Xapian.
 	 *
 	 *  @param dir directory holding quartz tables
 	 */
@@ -291,12 +312,12 @@ class QuartzWritableDatabase : public Xapian::Database::Internal {
 	bool term_exists(const string & tname) const;
 	bool has_positions() const;
 
-	LeafPostList * do_open_post_list(const string & tname) const;
-	LeafTermList * open_term_list(Xapian::docid did) const;
+	LeafPostList * open_post_list(const string & tname) const;
+	TermList * open_term_list(Xapian::docid did) const;
 	Xapian::Document::Internal * open_document(Xapian::docid did, bool lazy = false) const;
 	PositionList * open_position_list(Xapian::docid did,
 					  const string & tname) const;
-	TermList * open_allterms() const;
+	TermList * open_allterms(const string & prefix) const;
 	//@}
 };
 
