@@ -93,7 +93,7 @@ RemoteDatabase::RemoteDatabase(int fd, Xapian::timeout timeout_,
 	string errmsg("Unknown protocol version ");
 	errmsg += om_tostring(protocol_major);
 	errmsg += '.';
-	errmsg += om_tostring(int(*p));
+	errmsg += om_tostring(protocol_minor);
 	errmsg += " ("STRINGIZE(XAPIAN_REMOTE_PROTOCOL_MAJOR_VERSION)"."STRINGIZE(XAPIAN_REMOTE_PROTOCOL_MINOR_VERSION)" supported)";
 	throw Xapian::NetworkError(errmsg, context);
     }
@@ -422,11 +422,19 @@ RemoteDatabase::send_message(message_type type, const string &message) const
 void
 RemoteDatabase::do_close()
 {
-    // We should only really call dtor_called() if we're writable.  In the
-    // constructor, we set transaction_state to TRANSACTION_UNIMPLEMENTED
-    // if we aren't writable, so test that here.
-    if (transaction_state != TRANSACTION_UNIMPLEMENTED) dtor_called();
-    link.do_close();
+    // In the constructor, we set transaction_state to
+    // TRANSACTION_UNIMPLEMENTED if we aren't writable so that we can check
+    // it here.
+    bool writable = (transaction_state != TRANSACTION_UNIMPLEMENTED);
+
+    // Only call dtor_called() if we're writable.
+    if (writable) dtor_called();
+
+    // If we're writable, wait for a confirmation of the close, so we know that
+    // changes have been written and flushed, and the database write lock
+    // released.  For the non-writable case, there's no need to wait, so don't
+    // slow down searching by waiting here.
+    link.do_close(writable);
 }
 
 void
