@@ -1,6 +1,6 @@
 /* xapian-compact.cc: Compact a flint database, or merge and compact several.
  *
- * Copyright (C) 2004,2005,2006,2007 Olly Betts
+ * Copyright (C) 2004,2005,2006,2007,2008 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -73,6 +73,12 @@ is_metainfo_key(const string & key)
     return key.size() == 1 && key[0] == '\0';
 }
 
+static inline bool
+is_user_metadata_key(const string & key)
+{
+    return key.size() > 1 && key[0] == '\0' && key[1] == '\xc0';
+}
+
 class PostlistCursor : private FlintCursor {
     Xapian::docid offset;
 
@@ -82,7 +88,7 @@ class PostlistCursor : private FlintCursor {
     Xapian::termcount tf, cf;
 
     PostlistCursor(FlintTable *in, Xapian::docid offset_)
-	: FlintCursor(in), offset(offset_)
+	: FlintCursor(in), offset(offset_), firstdid(0)
     {
 	find_entry("");
 	next();
@@ -102,6 +108,7 @@ class PostlistCursor : private FlintCursor {
 	tag = current_tag;
 	tf = cf = 0;
 	if (is_metainfo_key(key)) return true;
+	if (is_user_metadata_key(key)) return true;
 	// Adjust key if this is *NOT* an initial chunk.
 	// key is: pack_string_preserving_sort(tname)
 	// plus optionally: pack_uint_preserving_sort(did)
@@ -188,11 +195,40 @@ merge_postlists(FlintTable * out, vector<Xapian::docid>::const_iterator offset,
 	}
     }
 
-    string tag = pack_uint(tot_off);
-    tag += pack_uint_last(tot_totlen);
-    out->add(string("", 1), tag);
+    {
+	string tag = pack_uint(tot_off);
+	tag += pack_uint_last(tot_totlen);
+	out->add(string("", 1), tag);
+    }
 
     string last_key;
+    {
+	// Merge user metadata.
+	string last_tag;
+	while (!pq.empty()) {
+	    PostlistCursor * cur = pq.top();
+	    const string& key = cur->key;
+	    if (!is_user_metadata_key(key)) break;
+
+	    const string & tag = cur->tag;
+	    if (key == last_key) {
+		if (tag != last_tag)
+		    cerr << "Warning: duplicate user metadata key with different tag value - picking arbitrary tag value" << endl;
+	    } else {
+		out->add(key, tag);
+		last_key = key;
+		last_tag = tag;
+	    }
+
+	    pq.pop();
+	    if (cur->next()) {
+		pq.push(cur);
+	    } else {
+		delete cur;
+	    }
+	}
+    }
+
     Xapian::termcount tf = 0, cf = 0; // Initialise to avoid warnings.
     vector<pair<Xapian::docid, string> > tags;
     while (true) {
@@ -201,6 +237,7 @@ merge_postlists(FlintTable * out, vector<Xapian::docid>::const_iterator offset,
 	    cur = pq.top();
 	    pq.pop();
 	}
+	Assert(cur == NULL || !is_user_metadata_key(cur->key));
 	if (cur == NULL || cur->key != last_key) {
 	    if (!tags.empty()) {
 		string first_tag = pack_uint(tf);
@@ -398,6 +435,7 @@ merge_spellings(FlintTable * out,
 	    // current_tag members must remain valid while we're merging their
 	    // tags, but we need to call next() on them all afterwards.
 	    vector<MergeCursor *> vec;
+	    vec.reserve(pq.size());
 
 	    while (true) {
 		cur->read_tag();
@@ -856,7 +894,7 @@ main(int argc, char **argv)
 	};
 
 	static const table_list tables[] = {
-	    // name	    type		compress_strategy	lazy
+	    // name	    type	compress_strategy	lazy
 	    { "postlist",   POSTLIST,	DONT_COMPRESS,		false },
 	    { "record",	    RECORD,	Z_DEFAULT_STRATEGY,	false },
 	    { "termlist",   TERMLIST,	Z_DEFAULT_STRATEGY,	false },
@@ -972,9 +1010,9 @@ main(int argc, char **argv)
 	    cout << endl;
 	}
 
-	// Copy the "iamflint" meta file over.
+	// Copy over the version file ("iamflint").
 	// FIXME: We may need to do something smarter that just copying an
-	// arbitrary meta file if the meta file format changes...
+	// arbitrary version file if the version file format changes...
 	string dest = destdir;
 	dest += "/iamflint.tmp";
 
@@ -990,8 +1028,8 @@ main(int argc, char **argv)
 		     << strerror(errno) << endl;
 		exit(1);
 	    }
-	    // metafile should be about 12 bytes, not > 1024!
-	    cerr << argv[0] << ": metafile '" << src << "' too large!"
+	    // Version file should be about 12 bytes, not > 1024!
+	    cerr << argv[0] << ": version file '" << src << "' too large!"
 		 << endl;
 	    exit(1);
 	}
@@ -1003,11 +1041,11 @@ main(int argc, char **argv)
 	}
 	output.close();
 
-	string meta = destdir;
-	meta += "/iamflint";
-	if (rename(dest.c_str(), meta.c_str()) == -1) {
+	string version = destdir;
+	version += "/iamflint";
+	if (rename(dest.c_str(), version.c_str()) == -1) {
 	    cerr << argv[0] << ": cannot rename '" << dest << "' to '"
-		 << meta << "': " << strerror(errno) << endl;
+		 << version << "': " << strerror(errno) << endl;
 	    exit(1);
 	}
     } catch (const Xapian::Error &error) {
