@@ -74,6 +74,11 @@ string om_tostring(const void * a);
 XAPIAN_VISIBILITY_DEFAULT
 bool file_exists(const string &fname);
 
+/** Return true if the directory dirname exists.
+ */
+XAPIAN_VISIBILITY_DEFAULT
+bool dir_exists(const string &dirname);
+
 /// Allow atoi to work directly on C++ strings.
 inline int atoi(const string &s) { return atoi(s.c_str()); }
 
@@ -100,4 +105,10 @@ inline int stat(const string &filename, struct stat *buf) {
     return stat(filename.c_str(), buf);
 }
 
+namespace Xapian {
+    namespace Internal {
+	bool within_DBL_EPSILON(double a, double b);
+    }
+}
+
 #endif /* OM_HGUARD_UTILS_H */
