@@ -1,7 +1,7 @@
 /** \file dbfactory.h
  * \brief Factory functions for constructing Database and WritableDatabase objects
  */
-/* Copyright (C) 2005,2006 Olly Betts
+/* Copyright (C) 2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,11 +25,15 @@
 #include <string>
 
 #include <xapian/types.h>
-#include <xapian/database.h>
+#include <xapian/deprecated.h>
 #include <xapian/version.h>
+#include <xapian/visibility.h>
 
 namespace Xapian {
 
+class Database;
+class WritableDatabase;
+
 namespace Auto {
 
 /** Construct a Database object for a stub database file.
@@ -39,36 +43,9 @@ namespace Auto {
  *
  * @param file  pathname of the stub database file.
  */
+XAPIAN_VISIBILITY_DEFAULT
 Database open_stub(const std::string &file);
 
-/** Construct a Database object for read-only access to a database.
- *
- * The appropriate database backend is chosen automatically.
- *
- * @deprecated	This function is deprecated - use the Xapian::Database(path)
- *		constructor instead.
- *
- * @param path  pathname of the file or directory containing the database.
- */
-XAPIAN_DEPRECATED(Database open(const std::string &path));
-inline Database open(const std::string &path) {
-    return Database(path);
-}
-
-/** Construct a WritableDatabase object for update access to a database.
- *
- * The appropriate database backend is chosen automatically.
- *
- * @deprecated	This function is deprecated - use the
- *		Xapian::WritableDatabase(path, action) constructor instead.
- *
- * @param path  pathname of the file or directory containing the database.
- */
-XAPIAN_DEPRECATED(WritableDatabase open(const std::string &path, int action));
-inline WritableDatabase open(const std::string &path, int action) {
-    return WritableDatabase(path, action);
-}
-
 }
 
 #ifdef XAPIAN_HAS_INMEMORY_BACKEND
@@ -78,75 +55,27 @@ namespace InMemory {
  *
  * A new, empty database is created for each call.
  */
+XAPIAN_VISIBILITY_DEFAULT
 WritableDatabase open();
 
 }
 #endif
 
-#ifdef XAPIAN_HAS_MUSCAT36_BACKEND
-namespace Muscat36 {
-
-/** Construct a Database object for read-only access to a Muscat36 DA database.
- *
- * @param R		pathname for the Record file.
- * @param T		pathname for the Term file.
- * @param heavy_duty	true if this DA database is "heavy-duty" (3 byte
- *			lengths); false if it's "flimsy" (2 byte lengths).
- */
-Database open_da(const std::string &R, const std::string &T, bool heavy_duty = true);
-
-/** Construct a Database object for read-only access to a Muscat36 DA database.
- *
- * This variants allows you to open a DA database with a values file.  Muscat36
- * didn't support values - support for them was added to Xapian for mostly to
- * allow testing during early development work.
- *
- * @param R		pathname for the Record file.
- * @param T		pathname for the Term file.
- * @param values	pathname for the Values file.
- * @param heavy_duty	true if this DA database is "heavy-duty" (3 byte
- *			lengths); false if it's "flimsy" (2 byte lengths).
- */
-Database open_da(const std::string &R, const std::string &T, const std::string &values, bool heavy_duty = true);
-
-/** Construct a Database object for read-only access to a Muscat36 DB database.
- *
- * It's easy to tell if a DB database is heavy-duty or flimsy (unlike with a
- * DA database), so this is automatically detected.
- *
- * @param DB		pathname for the DB file.
- * @param cache_size	size of the Btree block cache.
- */
-Database open_db(const std::string &DB, size_t cache_size = 30);
-
-/** Construct a Database object for read-only access to a Muscat36 DB database.
- *
- * This variants allows you to open a DB database with a values file.  Muscat36
- * didn't support values - support for them was added to Xapian for mostly to
- * allow testing during early development work.
- *
- * It's easy to tell if a DB database is heavy-duty or flimsy (unlike with a
- * DA database), so this is automatically detected.
- *
- * @param DB		pathname for the DB file.
- * @param values	pathname for the Values file.
- * @param cache_size	size of the Btree block cache.
- */
-Database open_db(const std::string &DB, const std::string &values, size_t cache_size = 30);
-
-}
-#endif
-
 #ifdef XAPIAN_HAS_QUARTZ_BACKEND
 namespace Quartz {
 
 /** Construct a Database object for read-only access to a Quartz database.
+ *
+ * The Quartz backend is deprecated - use the Flint backend instead.
  *
  * @param dir  pathname of the directory containing the database.
  */
-Database open(const std::string &dir);
+XAPIAN_VISIBILITY_DEFAULT
+XAPIAN_DEPRECATED(Database open(const std::string &dir));
 
 /** Construct a Database object for update access to a Quartz database.
+ *
+ * The Quartz backend is deprecated - use the Flint backend instead.
  *
  * @param dir		pathname of the directory containing the database.
  * @param action	determines handling of existing/non-existing database:
@@ -164,8 +93,9 @@ Database open(const std::string &dir);
  *			8192 bytes.  This parameter is ignored when opening an
  *			existing database.
  */
-WritableDatabase
-open(const std::string &dir, int action, int block_size = 8192);
+XAPIAN_VISIBILITY_DEFAULT
+XAPIAN_DEPRECATED(WritableDatabase
+open(const std::string &dir, int action, int block_size = 8192));
 
 }
 #endif
@@ -177,6 +107,7 @@ namespace Flint {
  *
  * @param dir  pathname of the directory containing the database.
  */
+XAPIAN_VISIBILITY_DEFAULT
 Database open(const std::string &dir);
 
 /** Construct a Database object for update access to a Flint database.
@@ -197,6 +128,7 @@ Database open(const std::string &dir);
  *			8192 bytes.  This parameter is ignored when opening an
  *			existing database.
  */
+XAPIAN_VISIBILITY_DEFAULT
 WritableDatabase
 open(const std::string &dir, int action, int block_size = 8192);
 
@@ -206,7 +138,6 @@ open(const std::string &dir, int action, int block_size = 8192);
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
 namespace Remote {
 
-//@{
 /** Construct a Database object for read-only access to a remote database
  *  accessed via a TCP connection.
  *
@@ -217,21 +148,38 @@ namespace Remote {
  * @param port		port number to connect to.
  * @param timeout	timeout in milliseconds.  If this timeout is exceeded
  *			for any individual operation on the remote database
- *			then Xapian::NetworkTimeoutError is thrown.  (Default
- *			is 10000ms, which is 10 seconds).
+ *			then Xapian::NetworkTimeoutError is thrown.  A timeout
+ *			of 0 means don't timeout.  (Default is 10000ms, which
+ *			is 10 seconds).
  * @param connect_timeout	timeout to use when connecting to the server.
  *				If this timeout is exceeded then
- *				Xapian::NetworkTimeoutError is throw.  (Default
- *				is to be the same as timeout).
+ *				Xapian::NetworkTimeoutError is thrown.  A
+ *				timeout of 0 means don't timeout.  (Default is
+ *				10000ms, which is 10 seconds).
  */
-Database open(const std::string &host, unsigned int port, Xapian::timeout timeout, Xapian::timeout connect_timeout);
-Database open(const std::string &host, unsigned int port, Xapian::timeout timeout = 10000);
-//@}
+XAPIAN_VISIBILITY_DEFAULT
+Database open(const std::string &host, unsigned int port, Xapian::timeout timeout = 10000, Xapian::timeout connect_timeout = 10000);
 
-//@{
-WritableDatabase open_writable(const std::string &host, unsigned int port, Xapian::timeout timeout, Xapian::timeout connect_timeout);
-WritableDatabase open_writable(const std::string &host, unsigned int port, Xapian::timeout timeout = 10000);
-//@}
+/** Construct a WritableDatabase object for update access to a remote database
+ *  accessed via a TCP connection.
+ *
+ * Access to the remote database is via a TCP connection to the specified
+ * host and port.
+ *
+ * @param host		hostname to connect to.
+ * @param port		port number to connect to.
+ * @param timeout	timeout in milliseconds.  If this timeout is exceeded
+ *			for any individual operation on the remote database
+ *			then Xapian::NetworkTimeoutError is thrown.  (Default
+ *			is 0, which means don't timeout).
+ * @param connect_timeout	timeout to use when connecting to the server.
+ *				If this timeout is exceeded then
+ *				Xapian::NetworkTimeoutError is thrown.  A
+ *				timeout of 0 means don't timeout.  (Default is
+ *				10000ms, which is 10 seconds).
+ */
+XAPIAN_VISIBILITY_DEFAULT
+WritableDatabase open_writable(const std::string &host, unsigned int port, Xapian::timeout timeout = 0, Xapian::timeout connect_timeout = 10000);
 
 /** Construct a Database object for read-only access to a remote database
  *  accessed via a program.
@@ -240,15 +188,31 @@ WritableDatabase open_writable(const std::string &host, unsigned int port, Xapia
  * communicating with it on stdin/stdout.
  *
  * @param program	the external program to run.
- * @param arguments	space-separated list of arguments to pass to program.
+ * @param args		space-separated list of arguments to pass to program.
  * @param timeout	timeout in milliseconds.  If this timeout is exceeded
  *			for any individual operation on the remote database
- *			then Xapian::NetworkTimeoutError is thrown (default is
- *			10000ms, which is 10 seconds).
+ *			then Xapian::NetworkTimeoutError is thrown.  A timeout
+ *			of 0 means don't timeout.  (Default is 10000ms, which
+ *			is 10 seconds).
  */
+XAPIAN_VISIBILITY_DEFAULT
 Database open(const std::string &program, const std::string &args, Xapian::timeout timeout = 10000);
 
-WritableDatabase open_writable(const std::string &program, const std::string &args, Xapian::timeout timeout = 10000);
+/** Construct a WritableDatabase object for update access to a remote database
+ *  accessed via a program.
+ *
+ * Access to the remote database is done by running an external program and
+ * communicating with it on stdin/stdout.
+ *
+ * @param program	the external program to run.
+ * @param args		space-separated list of arguments to pass to program.
+ * @param timeout	timeout in milliseconds.  If this timeout is exceeded
+ *			for any individual operation on the remote database
+ *			then Xapian::NetworkTimeoutError is thrown.  (Default
+ *			is 0, which means don't timeout).
+ */
+XAPIAN_VISIBILITY_DEFAULT
+WritableDatabase open_writable(const std::string &program, const std::string &args, Xapian::timeout timeout = 0);
 
 }
 #endif
