@@ -989,9 +989,15 @@ static void generate_integer_test(struct generator * g, struct node * p, char *
 static void generate_call(struct generator * g, struct node * p) {
 
     g->V[0] = p->name;
-    wp(g, "~{int ret = ~V0(~Z);~C"
-          "~Mif (ret == 0) ~f~N"
-          "~Mif (ret < 0) return ret;~N~}", p);
+    wp(g, "~{int ret = ~V0(~Z);~C", p);
+    if (g->failure_keep_count == 0 && g->failure_label == x_return) {
+	/* Combine the two tests in this special case for better optimisation
+	 * and clearer generated code. */
+	wp(g, "~Mif (ret <= 0) return ret;~N~}", p);
+    } else {
+	wp(g, "~Mif (ret == 0) ~f~N"
+	      "~Mif (ret < 0) return ret;~N~}", p);
+    }
 }
 
 static void generate_grouping(struct generator * g, struct node * p, int complement) {
@@ -1015,11 +1021,31 @@ static void generate_namedstring(struct generator * g, struct node * p) {
 
 static void generate_literalstring(struct generator * g, struct node * p) {
     symbol * b = p->literalstring;
-    g->S[0] = p->mode == m_forward ? "" : "_b";
-    g->I[0] = SIZE(b);
-    g->L[0] = b;
+    if (SIZE(b) == 1 && *b < 128) {
+	/* It's quite common to compare with a single ASCII character literal
+	 * string, so just inline the simpler code for this case rather than
+	 * making a function call. */
+	char buf[8];
+	if (*b < 32 || *b == 127 || *b == '\'' || *b == '\\') {
+	    sprintf(buf, "%d", (int)*b);
+	} else {
+	    sprintf(buf, "'%c'", (char)*b);
+	}
+	g->S[0] = buf;
+	if (p->mode == m_forward) {
+	    wp(g, "~Mif (c == l || p[c] != ~S0) ~f~N"
+		  "~Mc++;~N", p);
+	} else {
+	    wp(g, "~Mif (c <= lb || p[c - 1] != ~S0) ~f~N"
+		  "~Mc--;~N", p);
+	}
+    } else {
+	g->S[0] = p->mode == m_forward ? "" : "_b";
+	g->I[0] = SIZE(b);
+	g->L[0] = b;
 
-    wp(g, "~Mif (!(eq_s~S0(~Z~I0, ~L0))) ~f~C", p);
+	wp(g, "~Mif (!(eq_s~S0(~Z~I0, ~L0))) ~f~C", p);
+    }
 }
 
 static void generate_define(struct generator * g, struct node * p) {
