@@ -2,7 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2003,2004 Olly Betts
+ * Copyright 2003,2004,2007 Olly Betts
+ * Copyright 2007 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -84,6 +85,7 @@ AndPostList::AndPostList(PostList *left_, PostList *right_,
 	  dbsize(dbsize_)
 {
     DEBUGCALL(MATCH, void, "AndPostList", left_ << ", " << right_ << ", " << matcher_ << ", " << dbsize_ << ", " << replacement);
+    if (l->get_termfreq_est() > r->get_termfreq_est()) swap(l, r);
     if (replacement) {
 	// Initialise the maxweights from the kids so we can avoid forcing
 	// a full maxweight recalc
@@ -120,6 +122,18 @@ Xapian::doccount
 AndPostList::get_termfreq_min() const
 {
     DEBUGCALL(MATCH, Xapian::doccount, "AndPostList::get_termfreq_min", "");
+    // To have minimum matching documents, sets of documents matching both
+    // components of the AND must be minimal in size, and maximally disjoint.
+    //
+    // The overlap is then given by:
+    //  = lower_bound(left) + lower_bound(right) - dbsize
+    Xapian::doccount lmin = l->get_termfreq_min();
+    Xapian::doccount sum = lmin + r->get_termfreq_min();
+    // If sum < lmin, then the calculation overflowed and the true value of
+    // sum must be > dbsize.
+    if (sum < lmin || sum > dbsize) {
+	RETURN(sum - dbsize);
+    }
     RETURN(0u);
 }
 
@@ -131,7 +145,7 @@ AndPostList::get_termfreq_est() const
     // P(l and r) = P(l) . P(r)
     double lest = static_cast<double>(l->get_termfreq_est());
     double rest = static_cast<double>(r->get_termfreq_est());
-    RETURN(static_cast<Xapian::doccount> (lest * rest / dbsize));
+    RETURN(static_cast<Xapian::doccount>(lest * rest / dbsize + 0.5));
 }
 
 Xapian::docid
