@@ -2,10 +2,10 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008 Olly Betts
  * Copyright 2003 Orange PCS Ltd
  * Copyright 2003 Sam Liddicott
- * Copyright 2007 Lemur Consulting Ltd
+ * Copyright 2007,2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -276,7 +276,7 @@ MultiMatch::getorrecalc_maxweight(PostList *pl)
 	recalculate_w_max = false;
     } else {
 	wt = pl->get_maxweight();
-	AssertParanoid(fabs(wt - pl->recalc_maxweight()) < 1e-9);
+	AssertEqDoubleParanoid(wt, pl->recalc_maxweight());
     }
     DEBUGLINE(MATCH, "max possible doc weight = " << wt);
     RETURN(wt);
@@ -396,18 +396,21 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
     recalculate_w_max = false;
 
     Xapian::doccount matches_upper_bound = pl->get_termfreq_max();
-    Xapian::doccount matches_lower_bound = pl->get_termfreq_min();
+    Xapian::doccount matches_lower_bound = 0;
     Xapian::doccount matches_estimated   = pl->get_termfreq_est();
 
+    if (mdecider == NULL && matchspy == NULL) {
+	// If we have a matcher decider or match spy, the lower bound must be
+	// set to 0 as we could discard all hits.  Otherwise set it to the
+	// minimum number of entries which the postlist could return.
+	matches_lower_bound = pl->get_termfreq_min();
+    }
+
     // Check if any results have been asked for (might just be wanting
     // maxweight).
     if (check_at_least == 0) {
 	delete pl;
-	if (mdecider != NULL || matchspy != NULL) {
-	    // Lower bound must be set to 0 as the match decider could discard
-	    // all hits.
-	    matches_lower_bound = 0;
-	} else if (collapse_key != Xapian::BAD_VALUENO) {
+	if (collapse_key != Xapian::BAD_VALUENO) {
 	    // Lower bound must be set to no more than 1, since it's possible
 	    // that all hits will be collapsed to a single hit.
 	    if (matches_lower_bound > 1) matches_lower_bound = 1;
@@ -430,6 +433,11 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
     Xapian::doccount duplicates_found = 0;
     Xapian::doccount documents_considered = 0;
 
+    // Number of documents considered by a decider or matchspy.
+    Xapian::doccount decider_considered = 0;
+    // Number of documents denied by the decider or matchspy.
+    Xapian::doccount decider_denied = 0;
+
     // We keep track of the number of null collapse values because this allows
     // us to provide a better value for matches_lower_bound if there are null
     // collapse values.
@@ -447,7 +455,8 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
     // Minimum weight an item must have to be worth considering.
     Xapian::weight min_weight = weight_cutoff;
 
-    Xapian::weight percent_factor = percent_cutoff / 100.0;
+    // Factor to multiply maximum weight seen by to get the cutoff weight.
+    Xapian::weight percent_cutoff_factor = percent_cutoff / 100.0;
 
     // Table of keys which have been seen already, for collapsing.
     map<string, pair<Xapian::Internal::MSetItem,Xapian::weight> > collapse_tab;
@@ -571,8 +580,16 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		    doc = temp;
 		}
 		Xapian::Document mydoc(doc.get());
-		if (matchspy && !matchspy->operator()(mydoc)) continue;
-		if (mdecider && !mdecider->operator()(mydoc)) continue;
+
+		++decider_considered;
+		if (matchspy && !matchspy->operator()(mydoc)) {
+		    ++decider_denied;
+		    continue;
+		}
+		if (mdecider && !mdecider->operator()(mydoc)) {
+		    ++decider_denied;
+		    continue;
+		}
 	    }
 	}
 
@@ -728,7 +745,7 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	if (wt > greatest_wt) {
 	    greatest_wt = wt;
 	    if (percent_cutoff) {
-		Xapian::weight w = wt * percent_factor;
+		Xapian::weight w = wt * percent_cutoff_factor;
 		if (w > min_weight) {
 		    min_weight = w;
 		    if (!is_heap) {
@@ -763,7 +780,17 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	vector<Xapian::Internal::MSetItem>::const_iterator best;
 	best = min_element(items.begin(), items.end(), mcmp);
 
-	if (termfreqandwts.size() > 1) {
+	unsigned int multiplier = db.internal.size();
+	Assert(multiplier != 0);
+	Xapian::doccount n = (best->did - 1) % multiplier; // which actual database
+	// If the top result is from a remote database, then we can
+	// just use the percentage scaling calculated for that
+	// database.
+	if (is_remote[n]) {
+	    RemoteSubMatch * rem_match;
+	    rem_match = static_cast<RemoteSubMatch*>(leaves[n].get());
+	    percent_scale = rem_match->get_percent_factor();
+	} else if (termfreqandwts.size() > 1) {
 	    Xapian::termcount matching_terms = 0;
 	    map<string,
 		Xapian::MSet::Internal::TermFreqAndWeight>::const_iterator i;
@@ -812,7 +839,7 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	    // Or we could just use a linear scan here instead.
 
 	    // trim the mset to the correct answer...
-	    Xapian::weight min_wt = percent_factor / percent_scale;
+	    Xapian::weight min_wt = percent_cutoff_factor / percent_scale;
 	    if (!is_heap) {
 		is_heap = true;
 		make_heap<vector<Xapian::Internal::MSetItem>::iterator,
@@ -861,12 +888,69 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	matches_lower_bound = matches_upper_bound = matches_estimated
 	    = docs_matched;
     } else {
+	Assert(matches_estimated >= matches_lower_bound);
+	Assert(matches_estimated <= matches_upper_bound);
+
+	// We can end up scaling the estimate more than once, so collect
+	// the scale factors and apply them in one go to avoid rounding
+	// more than once.
+	double estimate_scale = 1.0;
+
+	if (collapse_key != Xapian::BAD_VALUENO) {
+	    // Lower bound must be set to no more than the number of null
+	    // collapse values seen plus the number of unique non-null collapse
+	    // values seen, since it's possible that all further hits will be
+	    // collapsed to values we've already seen.
+	    DEBUGLINE(MATCH, "Adjusting bounds due to collapse_key");
+	    matches_lower_bound = null_collapse_values + collapse_tab.size();
+
+	    // The estimate for the number of hits can be modified by
+	    // multiplying it by the rate at which we've been finding
+	    // unique documents.
+	    if (documents_considered > 0) {
+		double unique = double(documents_considered - duplicates_found);
+		double unique_rate = unique / double(documents_considered);
+		estimate_scale *= unique_rate;
+	    }
+
+	    // We can safely reduce the upper bound by the number of
+	    // duplicates we've seen.
+	    matches_upper_bound -= duplicates_found;
+
+	    DEBUGLINE(MATCH, "matches_lower_bound=" << matches_lower_bound <<
+		      ", matches_estimated=" << matches_estimated <<
+		      "*" << estimate_scale <<
+		      ", matches_upper_bound=" << matches_upper_bound);
+	}
+
+	if (matchspy || mdecider) {
+	    if (collapse_key == Xapian::BAD_VALUENO && !percent_cutoff) {
+		// We're not collapsing or doing a percentage cutoff, so
+		// docs_matched is a lower bound on the total number of matches.
+		matches_lower_bound = max(docs_matched, matches_lower_bound);
+	    }
+
+	    // The estimate for the number of hits can be modified by
+	    // multiplying it by the rate at which the match decider has
+	    // been accepting documents.
+	    if (decider_considered > 0) {
+		double accept = double(decider_considered - decider_denied);
+		double accept_rate = accept / double(decider_considered);
+		estimate_scale *= accept_rate;
+	    }
+
+	    // If a document is denied by a match decider, it is not possible
+	    // for it to found to be a duplicate, so it is safe to also reduce
+	    // the upper bound by the number of documents denied by a match
+	    // decider.
+	    matches_upper_bound -= decider_denied;
+	}
+
 	if (percent_cutoff) {
-	    // another approach: Xapian::doccount new_est = items.size() * (1 - percent_factor) / (1 - min_weight / greatest_wt);
-	    Xapian::doccount new_est;
-	    new_est = Xapian::doccount((1 - percent_factor) * matches_estimated);
-	    matches_estimated = max(size_t(new_est), items.size());
-	    // and another: items.size() + (1 - greatest_wt * percent_factor / min_weight) * (matches_estimated - items.size());
+	    estimate_scale *= (1.0 - percent_cutoff_factor);
+	    // another approach:
+	    // Xapian::doccount new_est = items.size() * (1 - percent_cutoff_factor) / (1 - min_weight / greatest_wt);
+	    // and another: items.size() + (1 - greatest_wt * percent_cutoff_factor / min_weight) * (matches_estimated - items.size());
 
 	    // Very likely an underestimate, but we can't really do better
 	    // without checking further matches...  Only possibility would be
@@ -876,33 +960,25 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	    matches_lower_bound = items.size();
 	    // matches_upper_bound could be reduced by the number of documents
 	    // which fail the min weight test
+	    DEBUGLINE(MATCH, "Adjusted bounds due to percent_cutoff (" <<
+		      percent_cutoff << "): now have matches_estimated=" <<
+		      matches_estimated << ", matches_lower_bound=" <<
+		      matches_lower_bound);
 	}
 
-	Assert(matches_estimated >= matches_lower_bound);
-	Assert(matches_estimated <= matches_upper_bound);
-
-	if (collapse_key != Xapian::BAD_VALUENO) {
-	    // Lower bound must be set to no more than the number of null
-	    // collapse values seen plus the number of unique non-null collapse
-	    // values seen, since it's possible that all further hits will be
-	    // collapsed to values we've already seen.
-	    matches_lower_bound = null_collapse_values + collapse_tab.size();
-
-	    // The estimate for the number of hits can be modified by
-	    // multiplying it by the rate at which we've been finding
-	    // duplicates.
-	    if (documents_considered > 0) {
-		double collapse_rate =
-			double(duplicates_found) / double(documents_considered);
-		matches_estimated -=
-			Xapian::doccount(double(matches_estimated) *
-					   collapse_rate);
-	    }
-
-	    // We can safely reduce the upper bound by the number of
-	    // duplicates we've seen.
-	    matches_upper_bound -= duplicates_found;
+	if (estimate_scale != 1.0) {
+	    matches_estimated =
+		Xapian::doccount(matches_estimated * estimate_scale + 0.5);
+	    if (matches_estimated < matches_lower_bound)
+	       	matches_estimated = matches_lower_bound;
+	}
 
+	if (collapse_key != Xapian::BAD_VALUENO || matchspy || mdecider) {
+	    DEBUGLINE(MATCH, "Clamping estimate between bounds: "
+		      "matches_lower_bound = " << matches_lower_bound <<
+		      ", matches_estimated = " << matches_estimated <<
+		      ", matches_upper_bound = " << matches_upper_bound);
+	    Assert(matches_lower_bound <= matches_upper_bound);
 	    matches_estimated = max(matches_estimated, matches_lower_bound);
 	    matches_estimated = min(matches_estimated, matches_upper_bound);
 	} else if (!percent_cutoff) {
@@ -959,7 +1035,7 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	!collapse_tab.empty()) {
 	// Nicked this formula from above, but for some reason percent_scale
 	// has since been multiplied by 100 so we take that into account
-	Xapian::weight min_wt = percent_factor / (percent_scale / 100);
+	Xapian::weight min_wt = percent_cutoff_factor / (percent_scale / 100);
 	vector<Xapian::Internal::MSetItem>::iterator i;
 	for (i = items.begin(); i != items.end() && !collapse_tab.empty(); ++i) {
 	    // Is this a collapsed hit?
