@@ -1,7 +1,7 @@
 /** @file valuerangepostlist.cc
  * @brief Return document ids matching a range test on a specified doc value.
  */
-/* Copyright 2007 Olly Betts
+/* Copyright 2007,2008 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,14 +20,21 @@
 
 #include <config.h>
 
+#include "valuerangepostlist.h"
+
 #include "autoptr.h"
 #include "omassert.h"
 #include "document.h"
+#include "leafpostlist.h"
 #include "utils.h"
-#include "valuerangepostlist.h"
 
 using namespace std;
 
+ValueRangePostList::~ValueRangePostList()
+{
+    delete alldocs_pl;
+}
+
 Xapian::doccount
 ValueRangePostList::get_termfreq_min() const
 {
@@ -103,17 +110,14 @@ PostList *
 ValueRangePostList::next(Xapian::weight)
 {
     Assert(db);
-    AssertParanoid(lastdocid == db->get_lastdocid());
-    while (current < lastdocid) {
-	try {
-	    if (++current == 0) break;
-	    AutoPtr<Xapian::Document::Internal> doc(db->open_document(current, true));
-	    string v = doc->get_value(valno);
-	    if (v >= begin && v <= end) return NULL;
-	} catch (const Xapian::DocNotFoundError &) {
-	    // That document doesn't exist.
-	    // FIXME: this could throw and catch a lot of exceptions!
-	}
+    if (!alldocs_pl) alldocs_pl = db->open_post_list(string());
+    alldocs_pl->skip_to(current + 1);
+    while (!alldocs_pl->at_end()) {
+	current = alldocs_pl->get_docid();
+	AutoPtr<Xapian::Document::Internal> doc(db->open_document(current, true));
+	string v = doc->get_value(valno);
+	if (v >= begin && v <= end) return NULL;
+	alldocs_pl->next();
     }
     db = NULL;
     return NULL;
@@ -136,25 +140,11 @@ ValueRangePostList::check(Xapian::docid did, Xapian::weight, bool &valid)
 	valid = true;
 	return NULL;
     }
-    AssertParanoid(lastdocid == db->get_lastdocid());
-    if (did > lastdocid) {
-	db = NULL;
-	valid = true;
-	return NULL;
-    }
-    try {
-	current = did;
-	AutoPtr<Xapian::Document::Internal> doc(db->open_document(current, true));
-	string v = doc->get_value(valno);
-	if (v >= begin && v <= end) {
-	    valid = true;
-	    return NULL;
-	}
-    } catch (const Xapian::DocNotFoundError &) {
-	// That document doesn't exist.
-	// FIXME: this could throw and catch a lot of exceptions!
-    }
-    valid = false;
+    AssertRelParanoid(did, <=, db->get_lastdocid());
+    current = did;
+    AutoPtr<Xapian::Document::Internal> doc(db->open_document(current, true));
+    string v = doc->get_value(valno);
+    valid = (v >= begin && v <= end);
     return NULL;
 }
 
