@@ -1,7 +1,7 @@
 /** @file remoteserver.cc
  *  @brief Xapian remote backend server base class
  */
-/* Copyright (C) 2006,2007 Olly Betts
+/* Copyright (C) 2006,2007,2008 Olly Betts
  * Copyright (C) 2006,2007 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or modify
@@ -74,12 +74,10 @@ RemoteServer::RemoteServer(const std::vector<std::string> &dbpaths,
 	    context += ' ';
 	    context += *i;
 	}
-
     } catch (const Xapian::Error &err) {
 	// Propagate the exception to the client.
 	send_message(REPLY_EXCEPTION, serialise_error(err));
-	// And rethrow it so our caller can log it and close the
-	// connection.
+	// And rethrow it so our caller can log it and close the connection.
 	throw;
     }
 
@@ -112,12 +110,8 @@ RemoteServer::RemoteServer(const std::vector<std::string> &dbpaths,
 
 RemoteServer::~RemoteServer()
 {
-    // wdb is either NULL or equal to db, so we shouldn't delete it.
-    if (wdb) {
-	delete wdb;
-    } else {
-	delete db;
-    }
+    delete db;
+    // wdb is either NULL or equal to db, so we shouldn't delete it too!
 
     map<string, Xapian::Weight*>::const_iterator i;
     for (i = wtschemes.begin(); i != wtschemes.end(); ++i) {
@@ -389,7 +383,7 @@ RemoteServer::msg_query(const string &message_in)
     if (*p < '0' || *p > '1') {
 	throw Xapian::NetworkError("bad message (sort_value_forward)");
     }
-    bool sort_value_forward(*p++ - '0');
+    bool sort_value_forward(*p++ != '0');
 
     int percent_cutoff = *p++;
     if (percent_cutoff < 0 || percent_cutoff > 100) {
