@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2004,2005,2006 Olly Betts
+ * Copyright 2004,2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,48 +23,31 @@
 #include <config.h>
 
 #include "safeerrno.h"
+#include "safefcntl.h"
 
 #include "tcpclient.h"
 #include <xapian/error.h>
 
-#include <netdb.h>
-#include <netinet/in.h>
-#include <netinet/tcp.h>
 #include <string.h>
-#ifdef __WIN32__
-# include <winsock2.h>
-#else
+#ifndef __WIN32__
+# include <netdb.h>
+# include <netinet/in.h>
+# include <netinet/tcp.h>
 # include <sys/socket.h>
+# include "safesysselect.h"
 #endif
 
-#ifdef HAVE_SYS_SELECT_H
-# include <sys/select.h>
-#else
-# include <sys/time.h>
-# include <sys/types.h>
-# include <unistd.h>
-#endif
-#include <string.h> // Solaris needs this as FDSET uses memset but fails to prototype it.
-
 #include "utils.h"
 
-TcpClient::TcpClient(std::string hostname, int port, int msecs_timeout_, int msecs_timeout_connect_)
-	: RemoteDatabase(get_remote_socket(hostname, port, msecs_timeout_connect_),
-		         msecs_timeout_, get_tcpcontext(hostname, port))
-{
-
-}
-
 std::string
-TcpClient::get_tcpcontext(std::string hostname, int port)
+TcpClient::get_tcpcontext(const std::string & hostname, int port)
 {
     return "remote:tcp(" + hostname + ":" + om_tostring(port) + ")";
 }
 
 int
-TcpClient::get_remote_socket(std::string hostname,
-			     int port,
-			     int msecs_timeout_connect_)
+TcpClient::open_socket(const std::string & hostname, int port,
+		       int msecs_timeout_connect)
 {
     // Note: can't use RemoteDatabase::timeout because it won't yet have
     // been initialised.
@@ -73,14 +56,25 @@ TcpClient::get_remote_socket(std::string hostname,
     struct hostent *host = gethostbyname(hostname.c_str());
 
     if (host == 0) {
-	throw Xapian::NetworkError(std::string("Couldn't resolve host ") +
-			     hostname, get_tcpcontext(hostname, port));
+	throw Xapian::NetworkError(std::string("Couldn't resolve host ") + hostname,
+		get_tcpcontext(hostname, port),
+#ifdef __WIN32__
+		socket_errno()
+#else
+		// "socket_errno()" is just errno on UNIX which is
+		// inappropriate here - if gethostbyname() returns NULL an
+		// error code is available in h_errno (with values
+		// incompatible with errno).  On Linux at least, if h_errno
+		// is < 0, then the error code *IS* in errno!
+		(h_errno < 0 ? errno : -h_errno)
+#endif
+		);
     }
 
     int socketfd = socket(PF_INET, SOCK_STREAM, 0);
 
     if (socketfd < 0) {
-	throw Xapian::NetworkError("Couldn't create socket", get_tcpcontext(hostname, port), errno);
+	throw Xapian::NetworkError("Couldn't create socket", get_tcpcontext(hostname, port), socket_errno());
     }
 
     struct sockaddr_in remaddr;
@@ -88,17 +82,26 @@ TcpClient::get_remote_socket(std::string hostname,
     remaddr.sin_port = htons(port);
     memcpy(&remaddr.sin_addr, host->h_addr, sizeof(remaddr.sin_addr));
 
-    if (fcntl(socketfd, F_SETFL, O_NDELAY) < 0) {
-	int saved_errno = errno; // note down in case close hits an error
+#ifdef __WIN32__
+    ULONG enabled = 1;
+    int rc = ioctlsocket(socketfd, FIONBIO, &enabled);
+#else
+    int rc = fcntl(socketfd, F_SETFL, O_NDELAY);
+#endif
+    if (rc < 0) {
+	int saved_errno = socket_errno(); // note down in case close hits an error
 	close(socketfd);
 	throw Xapian::NetworkError("Couldn't set O_NDELAY", get_tcpcontext(hostname,  port), saved_errno);
     }
 
     {
 	int optval = 1;
+	// 4th argument might need to be void* or char* - cast it to char*
+	// since C++ allows implicit conversion to void* but not from void*.
 	if (setsockopt(socketfd, IPPROTO_TCP, TCP_NODELAY,
-		       reinterpret_cast<void *>(&optval), sizeof(optval)) < 0) {
-	    int saved_errno = errno; // note down in case close hits an error
+		       reinterpret_cast<char *>(&optval),
+		       sizeof(optval)) < 0) {
+	    int saved_errno = socket_errno(); // note down in case close hits an error
 	    close(socketfd);
 	    throw Xapian::NetworkError("Couldn't set TCP_NODELAY", get_tcpcontext(hostname,  port), saved_errno);
 	}
@@ -108,8 +111,12 @@ TcpClient::get_remote_socket(std::string hostname,
 			 sizeof(remaddr));
 
     if (retval < 0) {
-	if (errno != EINPROGRESS) {
-	    int saved_errno = errno; // note down in case close hits an error
+#ifdef __WIN32__
+	if (WSAGetLastError() != WSAEWOULDBLOCK) {
+#else
+	if (socket_errno() != EINPROGRESS) {
+#endif
+	    int saved_errno = socket_errno(); // note down in case close hits an error
 	    close(socketfd);
 	    throw Xapian::NetworkError("Couldn't connect", get_tcpcontext(hostname, port), saved_errno);
 	}
@@ -120,8 +127,8 @@ TcpClient::get_remote_socket(std::string hostname,
 	FD_SET(socketfd, &fdset);
 
 	struct timeval tv;
-	tv.tv_sec = msecs_timeout_connect_ / 1000;
-	tv.tv_usec = msecs_timeout_connect_ % 1000 * 1000;
+	tv.tv_sec = msecs_timeout_connect / 1000;
+	tv.tv_usec = msecs_timeout_connect % 1000 * 1000;
 
 	retval = select(socketfd + 1, 0, &fdset, &fdset, &tv);
 
@@ -133,12 +140,13 @@ TcpClient::get_remote_socket(std::string hostname,
 	int err = 0;
 	SOCKLEN_T len = sizeof(err);
 
-	/* On Solaris 5.6, the fourth argument is char *. */
+	// 4th argument might need to be void* or char* - cast it to char*
+	// since C++ allows implicit conversion to void* but not from void*.
 	retval = getsockopt(socketfd, SOL_SOCKET, SO_ERROR,
-			    reinterpret_cast<void *>(&err), &len);
+			    reinterpret_cast<char *>(&err), &len);
 
 	if (retval < 0) {
-	    int saved_errno = errno; // note down in case close hits an error
+	    int saved_errno = socket_errno(); // note down in case close hits an error
 	    close(socketfd);
 	    throw Xapian::NetworkError("Couldn't get socket options", get_tcpcontext(hostname, port), saved_errno);
 	}
@@ -148,8 +156,12 @@ TcpClient::get_remote_socket(std::string hostname,
 	}
     }
 
+#ifdef __WIN32__
+    enabled = 0;
+    ioctlsocket(socketfd, FIONBIO, &enabled);
+#else
     fcntl(socketfd, F_SETFL, 0);
-
+#endif
     return socketfd;
 }
 
