@@ -3,11 +3,11 @@
 */
 /* First off, code is included which follows the "include" declaration
 ** in the input file. */
-#line 1 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 
 /* queryparser.lemony: build a Xapian::Query object from a user query string.
  *
- * Copyright (C) 2004,2005,2006,2007 Olly Betts
+ * Copyright (C) 2004,2005,2006,2007,2008 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -339,40 +339,40 @@ Query *
 Term::as_wildcarded_query(State * state_) const
 {
     Database db = state_->get_database();
-    Query * q = new Query;
+    vector<Query> subqs;
     list<string>::const_iterator piter;
     for (piter = prefixes.begin(); piter != prefixes.end(); ++piter) {
 	string root = *piter;
 	root += name;
 	TermIterator t = db.allterms_begin(root);
 	while (t != db.allterms_end(root)) {
-	    add_to_query(q, Query::OP_OR, Query(*t, 1, pos));
+	    subqs.push_back(Query(*t, 1, pos));
 	    ++t;
 	}
     }
     delete this;
-    return q;
+    return new Query(Query::OP_OR, subqs.begin(), subqs.end());
 }
 
 Query *
 Term::as_partial_query(State * state_) const
 {
     Database db = state_->get_database();
-    Query * q = new Query;
+    vector<Query> subqs;
     list<string>::const_iterator piter;
     for (piter = prefixes.begin(); piter != prefixes.end(); ++piter) {
 	string root = *piter;
 	root += name;
 	TermIterator t = db.allterms_begin(root);
 	while (t != db.allterms_end(root)) {
-	    add_to_query(q, Query::OP_OR, Query(*t, 1, pos));
+	    subqs.push_back(Query(*t, 1, pos));
 	    ++t;
 	}
 	// Add the term, as it would normally be handled, as an alternative.
-	add_to_query(q, Query::OP_OR, Query(make_term(*piter), 1, pos));
+	subqs.push_back(Query(make_term(*piter), 1, pos));
     }
     delete this;
-    return q;
+    return new Query(Query::OP_OR, subqs.begin(), subqs.end());
 }
 
 inline bool
@@ -585,7 +585,7 @@ main_lex_loop:
     enum {
 	DEFAULT, IN_QUOTES, IN_PREFIXED_QUOTES, IN_PHRASED_TERM, IN_GROUP
     } mode = DEFAULT;
-    while (it != end) {
+    while (it != end && !state.error) {
 	bool last_was_operator = false;
 	if (false) {
 just_had_operator:
@@ -1031,10 +1031,12 @@ phrased_term:
 	}
     }
 done:
-    // Implicitly close any unclosed quotes...
-    if (mode == IN_QUOTES || mode == IN_PREFIXED_QUOTES)
-	Parse(pParser, QUOTE, NULL, &state);
-    Parse(pParser, 0, NULL, &state);
+    if (!state.error) {
+	// Implicitly close any unclosed quotes...
+	if (mode == IN_QUOTES || mode == IN_PREFIXED_QUOTES)
+	    Parse(pParser, QUOTE, NULL, &state);
+	Parse(pParser, 0, NULL, &state);
+    }
     ParseFree(pParser);
 
     errmsg = state.error;
@@ -1069,7 +1071,7 @@ struct ProbQuery {
 };
 
 class TermGroup {
-    list<Term *> terms;
+    vector<Term *> terms;
 
   public:
     TermGroup() { }
@@ -1092,7 +1094,7 @@ class TermGroup {
      *  compile-time error - you must allocate this object with new.
      */
     ~TermGroup() {
-	list<Term*>::const_iterator i;
+	vector<Term*>::const_iterator i;
 	for (i = terms.begin(); i != terms.end(); ++i) {
 	    delete *i;
 	}
@@ -1102,21 +1104,36 @@ class TermGroup {
 Query *
 TermGroup::as_group(State *state) const
 {
-    Query * query = NULL;
     Query::op default_op = state->default_op();
+    vector<Query> subqs;
+    subqs.reserve(terms.size());
     if (state->flags & QueryParser::FLAG_AUTO_MULTIWORD_SYNONYMS) {
 	// Check for multi-word synonyms.
 	Database db = state->get_database();
 
 	string key;
-	list<Term*>::const_iterator begin = terms.begin();
-	list<Term*>::const_iterator i = begin;
+	vector<Term*>::const_iterator begin = terms.begin();
+	vector<Term*>::const_iterator i = begin;
 	while (i != terms.end()) {
+	    TermIterator synkey(db.synonym_keys_begin((*i)->name));
+	    TermIterator synend(db.synonym_keys_end((*i)->name));
+	    if (synkey == synend) {
+		// No multi-synonym matches.
+		if (state->is_stopword(*i)) {
+		    state->add_to_stoplist(*i);
+		} else {
+		    subqs.push_back((*i)->get_query_with_auto_synonyms());
+		}
+		begin = ++i;
+		continue;
+	    }
 	    key.resize(0);
 	    while (i != terms.end()) {
 		if (!key.empty()) key += ' ';
 		key += (*i)->name;
 		++i;
+		synkey.skip_to(key);
+		if (synkey == synend || !startswith(*synkey, key)) break;
 	    }
 	    // Greedily try to match as many consecutive words as possible.
 	    TermIterator syn, end;
@@ -1132,49 +1149,52 @@ TermGroup::as_group(State *state) const
 		if (state->is_stopword(*i)) {
 		    state->add_to_stoplist(*i);
 		} else {
-		    add_to_query(query, default_op,
-				 (*i)->get_query_with_auto_synonyms());
+		    subqs.push_back((*i)->get_query_with_auto_synonyms());
 		}
 		begin = ++i;
 		continue;
 	    }
 
-	    Query * q = NULL;
-	    list<Term*>::const_iterator j;
+	    vector<Query> subqs2;
+	    vector<Term*>::const_iterator j;
 	    for (j = begin; j != i; ++j) {
 		if (state->is_stopword(*j)) {
 		    state->add_to_stoplist(*j);
 		} else {
-		    add_to_query(q, default_op, (*j)->get_query());
+		    subqs2.push_back((*j)->get_query());
 		}
 	    }
+	    Query q_original_terms(default_op, subqs2.begin(), subqs2.end());
+	    subqs2.clear();
 
 	    // Use the position of the first term for the synonyms.
 	    Xapian::termpos pos = (*begin)->pos;
 	    begin = i;
 	    while (syn != end) {
-		add_to_query(q, Query::OP_OR, Query(*syn, 1, pos));
+		subqs2.push_back(Query(*syn, 1, pos));
 		++syn;
 	    }
-	    add_to_query(query, default_op, q);
+	    Query q_synonym_terms(Query::OP_OR, subqs2.begin(), subqs2.end());
+	    subqs2.clear();
+	    subqs.push_back(Query(Query::OP_OR,
+				  q_original_terms, q_synonym_terms));
 	}
     } else {
-	list<Term*>::const_iterator i;
+	vector<Term*>::const_iterator i;
 	for (i = terms.begin(); i != terms.end(); ++i) {
 	    if (state->is_stopword(*i)) {
 		state->add_to_stoplist(*i);
 	    } else {
-		add_to_query(query, default_op,
-			     (*i)->get_query_with_auto_synonyms());
+	    	subqs.push_back((*i)->get_query_with_auto_synonyms());
 	    }
 	}
     }
     delete this;
-    return query;
+    return new Query(default_op, subqs.begin(), subqs.end());
 }
 
 class TermList {
-    list<Term *> terms;
+    vector<Term *> terms;
     size_t window;
 
     /** Keep track of whether the terms added all have the same list of
@@ -1215,7 +1235,6 @@ class TermList {
     /// Convert to a query using the given operator and window size.
     Query * as_opwindow_query(Query::op op, Xapian::termcount w_delta) const {
 	Query * q = NULL;
-	// Call terms.size() just once since std::list::size() may be O(n).
 	size_t n_terms = terms.size();
 	Xapian::termcount w = w_delta + terms.size();
 	if (uniform_prefixes) {
@@ -1223,7 +1242,7 @@ class TermList {
 	    for (piter = prefixes.begin(); piter != prefixes.end(); ++piter) {
 		vector<Query> subqs;
 		subqs.reserve(n_terms);
-		list<Term *>::const_iterator titer;
+		vector<Term *>::const_iterator titer;
 		for (titer = terms.begin(); titer != terms.end(); ++titer) {
 		    Term * t = *titer;
 		    subqs.push_back(Query(t->make_term(*piter), 1, t->pos));
@@ -1234,7 +1253,7 @@ class TermList {
 	} else {
 	    vector<Query> subqs;
 	    subqs.reserve(n_terms);
-	    list<Term *>::const_iterator titer;
+	    vector<Term *>::const_iterator titer;
 	    for (titer = terms.begin(); titer != terms.end(); ++titer) {
 		subqs.push_back((*titer)->get_query());
 	    }
@@ -1280,7 +1299,7 @@ class TermList {
      *  compile-time error - you must allocate this object with new.
      */
     ~TermList() {
-	list<Term *>::const_iterator t;
+	vector<Term *>::const_iterator t;
 	for (t = terms.begin(); t != terms.end(); ++t) {
 	    delete *t;
 	}
@@ -1300,7 +1319,7 @@ class TermList {
 	delete B;\
     } while (0)
 
-#line 1305 "queryparser/queryparser_internal.cc"
+#line 1324 "queryparser/queryparser_internal.cc"
 /* Next is all token values, in a form suitable for use by makeheaders.
 ** This section will be null unless lemon is run with the -m switch.
 */
@@ -1728,9 +1747,9 @@ static void yy_destructor(YYCODETYPE yymajor, YYMINORTYPE *yypminor){
     case 19:
     case 20:
     case 21:
-#line 1300 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1319 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {delete (yypminor->yy0);}
-#line 1735 "queryparser/queryparser_internal.cc"
+#line 1754 "queryparser/queryparser_internal.cc"
       break;
     case 24:
     case 25:
@@ -1738,28 +1757,28 @@ static void yy_destructor(YYCODETYPE yymajor, YYMINORTYPE *yypminor){
     case 28:
     case 30:
     case 31:
-#line 1371 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1390 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {delete (yypminor->yy13);}
-#line 1745 "queryparser/queryparser_internal.cc"
+#line 1764 "queryparser/queryparser_internal.cc"
       break;
     case 27:
     case 29:
-#line 1464 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1483 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {delete (yypminor->yy9);}
-#line 1751 "queryparser/queryparser_internal.cc"
+#line 1770 "queryparser/queryparser_internal.cc"
       break;
     case 32:
     case 33:
     case 35:
     case 36:
-#line 1665 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1684 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {(yypminor->yy1)->destroy();}
-#line 1759 "queryparser/queryparser_internal.cc"
+#line 1778 "queryparser/queryparser_internal.cc"
       break;
     case 34:
-#line 1699 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1718 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {(yypminor->yy60)->destroy();}
-#line 1764 "queryparser/queryparser_internal.cc"
+#line 1783 "queryparser/queryparser_internal.cc"
       break;
     default:  break;   /* If no destructor action specified: do nothing */
   }
@@ -1991,7 +2010,7 @@ static void yy_reduce(
   **     break;
   */
       case 0:
-#line 1353 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1372 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     // Save the parsed query in the State structure so we can return it.
     if (yymsp[0].minor.yy13) {
@@ -2001,30 +2020,30 @@ static void yy_reduce(
 	state->query = Query();
     }
 }
-#line 2006 "queryparser/queryparser_internal.cc"
+#line 2025 "queryparser/queryparser_internal.cc"
         break;
       case 1:
-#line 1363 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1382 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     // Handle a query string with no terms in.
     state->query = Query();
 }
-#line 2014 "queryparser/queryparser_internal.cc"
+#line 2033 "queryparser/queryparser_internal.cc"
         break;
       case 2:
       case 8:
-#line 1374 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1393 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 { yygotominor.yy13 = yymsp[0].minor.yy13; }
-#line 2020 "queryparser/queryparser_internal.cc"
+#line 2039 "queryparser/queryparser_internal.cc"
         break;
       case 3:
-#line 1377 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1396 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy13, yymsp[-2].minor.yy13, Query::OP_AND, yymsp[0].minor.yy13, "AND");   yy_destructor(4,&yymsp[-1].minor);
 }
-#line 2026 "queryparser/queryparser_internal.cc"
+#line 2045 "queryparser/queryparser_internal.cc"
         break;
       case 4:
-#line 1379 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1398 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     // 'NOT foo' -> '<alldocuments> NOT foo'
     if (!yymsp[-2].minor.yy13 && (state->flags & QueryParser::FLAG_PURE_NOT)) {
@@ -2033,39 +2052,39 @@ static void yy_reduce(
     BOOL_OP_TO_QUERY(yygotominor.yy13, yymsp[-2].minor.yy13, Query::OP_AND_NOT, yymsp[0].minor.yy13, "NOT");
   yy_destructor(5,&yymsp[-1].minor);
 }
-#line 2038 "queryparser/queryparser_internal.cc"
+#line 2057 "queryparser/queryparser_internal.cc"
         break;
       case 5:
-#line 1388 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1407 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy13, yymsp[-3].minor.yy13, Query::OP_AND_NOT, yymsp[0].minor.yy13, "AND NOT");   yy_destructor(4,&yymsp[-2].minor);
   yy_destructor(5,&yymsp[-1].minor);
 }
-#line 2045 "queryparser/queryparser_internal.cc"
+#line 2064 "queryparser/queryparser_internal.cc"
         break;
       case 6:
-#line 1391 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1410 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy13, yymsp[-2].minor.yy13, Query::OP_OR, yymsp[0].minor.yy13, "OR");   yy_destructor(2,&yymsp[-1].minor);
 }
-#line 2051 "queryparser/queryparser_internal.cc"
+#line 2070 "queryparser/queryparser_internal.cc"
         break;
       case 7:
-#line 1394 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1413 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy13, yymsp[-2].minor.yy13, Query::OP_XOR, yymsp[0].minor.yy13, "XOR");   yy_destructor(3,&yymsp[-1].minor);
 }
-#line 2057 "queryparser/queryparser_internal.cc"
+#line 2076 "queryparser/queryparser_internal.cc"
         break;
       case 9:
-#line 1403 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1422 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     // Set the argument to NULL, which enables the bool_arg-using rules in
     // expr above to report uses of AND, OR, etc which don't have two
     // arguments.
     yygotominor.yy13 = NULL;
 }
-#line 2067 "queryparser/queryparser_internal.cc"
+#line 2086 "queryparser/queryparser_internal.cc"
         break;
       case 10:
-#line 1415 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1434 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy13 = yymsp[0].minor.yy9->query;
     yymsp[0].minor.yy9->query = NULL;
@@ -2104,19 +2123,19 @@ static void yy_reduce(
     // FIXME what if yygotominor.yy13 && yygotominor.yy13->empty() (all terms are stopwords)?
     delete yymsp[0].minor.yy9;
 }
-#line 2109 "queryparser/queryparser_internal.cc"
+#line 2128 "queryparser/queryparser_internal.cc"
         break;
       case 11:
       case 29:
       case 31:
-#line 1454 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1473 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy13 = yymsp[0].minor.yy13;
 }
-#line 2118 "queryparser/queryparser_internal.cc"
+#line 2137 "queryparser/queryparser_internal.cc"
         break;
       case 12:
-#line 1466 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1485 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     Query range;
     Xapian::valueno valno = state->value_range(range, yymsp[-1].minor.yy0, yymsp[0].minor.yy0);
@@ -2127,10 +2146,10 @@ static void yy_reduce(
     yygotominor.yy9 = new ProbQuery;
     yygotominor.yy9->filter[filter_group_id(valno)] = range;
 }
-#line 2132 "queryparser/queryparser_internal.cc"
+#line 2151 "queryparser/queryparser_internal.cc"
         break;
       case 13:
-#line 1477 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1496 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     Query range;
     Xapian::valueno valno = state->value_range(range, yymsp[-1].minor.yy0, yymsp[0].minor.yy0);
@@ -2142,28 +2161,28 @@ static void yy_reduce(
     Query & q = yygotominor.yy9->filter[filter_group_id(valno)];
     q = Query(Query::OP_OR, q, range);
 }
-#line 2147 "queryparser/queryparser_internal.cc"
+#line 2166 "queryparser/queryparser_internal.cc"
         break;
       case 14:
-#line 1489 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1508 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = new ProbQuery;
     yygotominor.yy9->query = yymsp[-1].minor.yy13;
     if (yymsp[0].minor.yy13) add_to_query(yygotominor.yy9->query, state->default_op(), yymsp[0].minor.yy13);
 }
-#line 2156 "queryparser/queryparser_internal.cc"
+#line 2175 "queryparser/queryparser_internal.cc"
         break;
       case 15:
-#line 1495 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1514 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = yymsp[-1].minor.yy9;
     // If yymsp[0].minor.yy13 is a stopword, there's nothing to do here.
     if (yymsp[0].minor.yy13) add_to_query(yygotominor.yy9->query, state->default_op(), yymsp[0].minor.yy13);
 }
-#line 2165 "queryparser/queryparser_internal.cc"
+#line 2184 "queryparser/queryparser_internal.cc"
         break;
       case 16:
-#line 1501 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1520 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = new ProbQuery;
     if (state->default_op() == Query::OP_AND) {
@@ -2173,10 +2192,10 @@ static void yy_reduce(
     }
   yy_destructor(8,&yymsp[-1].minor);
 }
-#line 2178 "queryparser/queryparser_internal.cc"
+#line 2197 "queryparser/queryparser_internal.cc"
         break;
       case 17:
-#line 1510 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1529 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = yymsp[-2].minor.yy9;
     if (state->default_op() == Query::OP_AND) {
@@ -2189,57 +2208,57 @@ static void yy_reduce(
     }
   yy_destructor(8,&yymsp[-1].minor);
 }
-#line 2194 "queryparser/queryparser_internal.cc"
+#line 2213 "queryparser/queryparser_internal.cc"
         break;
       case 18:
-#line 1522 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1541 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = new ProbQuery;
     yygotominor.yy9->hate = yymsp[0].minor.yy13;
   yy_destructor(9,&yymsp[-1].minor);
 }
-#line 2203 "queryparser/queryparser_internal.cc"
+#line 2222 "queryparser/queryparser_internal.cc"
         break;
       case 19:
-#line 1527 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1546 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = yymsp[-2].minor.yy9;
     add_to_query(yygotominor.yy9->hate, Query::OP_OR, yymsp[0].minor.yy13);
   yy_destructor(9,&yymsp[-1].minor);
 }
-#line 2212 "queryparser/queryparser_internal.cc"
+#line 2231 "queryparser/queryparser_internal.cc"
         break;
       case 20:
-#line 1532 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1551 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = new ProbQuery;
     yygotominor.yy9->hate = new Query(yymsp[0].minor.yy0->get_query());
     delete yymsp[0].minor.yy0;
   yy_destructor(9,&yymsp[-1].minor);
 }
-#line 2222 "queryparser/queryparser_internal.cc"
+#line 2241 "queryparser/queryparser_internal.cc"
         break;
       case 21:
-#line 1538 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1557 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = yymsp[-2].minor.yy9;
     add_to_query(yygotominor.yy9->hate, Query::OP_OR, yymsp[0].minor.yy0->get_query());
     delete yymsp[0].minor.yy0;
   yy_destructor(9,&yymsp[-1].minor);
 }
-#line 2232 "queryparser/queryparser_internal.cc"
+#line 2251 "queryparser/queryparser_internal.cc"
         break;
       case 22:
-#line 1544 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1563 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = new ProbQuery;
     yygotominor.yy9->filter[yymsp[0].minor.yy0->get_filter_group_id()] = yymsp[0].minor.yy0->get_query();
     delete yymsp[0].minor.yy0;
 }
-#line 2241 "queryparser/queryparser_internal.cc"
+#line 2260 "queryparser/queryparser_internal.cc"
         break;
       case 23:
-#line 1550 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1569 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = yymsp[-1].minor.yy9;
     // We OR filters with the same prefix...
@@ -2247,10 +2266,10 @@ static void yy_reduce(
     q = Query(Query::OP_OR, q, yymsp[0].minor.yy0->get_query());
     delete yymsp[0].minor.yy0;
 }
-#line 2252 "queryparser/queryparser_internal.cc"
+#line 2271 "queryparser/queryparser_internal.cc"
         break;
       case 24:
-#line 1558 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1577 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     // LOVE BOOLEAN_FILTER(yymsp[0].minor.yy0) is just the same as BOOLEAN_FILTER
     yygotominor.yy9 = new ProbQuery;
@@ -2258,10 +2277,10 @@ static void yy_reduce(
     delete yymsp[0].minor.yy0;
   yy_destructor(8,&yymsp[-1].minor);
 }
-#line 2263 "queryparser/queryparser_internal.cc"
+#line 2282 "queryparser/queryparser_internal.cc"
         break;
       case 25:
-#line 1565 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1584 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     // LOVE BOOLEAN_FILTER(yymsp[0].minor.yy0) is just the same as BOOLEAN_FILTER
     yygotominor.yy9 = yymsp[-2].minor.yy9;
@@ -2271,23 +2290,23 @@ static void yy_reduce(
     delete yymsp[0].minor.yy0;
   yy_destructor(8,&yymsp[-1].minor);
 }
-#line 2276 "queryparser/queryparser_internal.cc"
+#line 2295 "queryparser/queryparser_internal.cc"
         break;
       case 26:
-#line 1580 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1599 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 { yygotominor.yy9 = yymsp[0].minor.yy9; }
-#line 2281 "queryparser/queryparser_internal.cc"
+#line 2300 "queryparser/queryparser_internal.cc"
         break;
       case 27:
-#line 1582 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1601 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = new ProbQuery;
     yygotominor.yy9->query = yymsp[0].minor.yy13;
 }
-#line 2289 "queryparser/queryparser_internal.cc"
+#line 2308 "queryparser/queryparser_internal.cc"
         break;
       case 28:
-#line 1596 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1615 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     if (state->is_stopword(yymsp[0].minor.yy0)) {
 	yygotominor.yy13 = NULL;
@@ -2297,117 +2316,117 @@ static void yy_reduce(
     }
     delete yymsp[0].minor.yy0;
 }
-#line 2302 "queryparser/queryparser_internal.cc"
+#line 2321 "queryparser/queryparser_internal.cc"
         break;
       case 30:
-#line 1615 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1634 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy13 = new Query(yymsp[0].minor.yy0->get_query_with_auto_synonyms());
     delete yymsp[0].minor.yy0;
 }
-#line 2310 "queryparser/queryparser_internal.cc"
+#line 2329 "queryparser/queryparser_internal.cc"
         break;
       case 32:
-#line 1632 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1651 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 { yygotominor.yy13 = yymsp[0].minor.yy0->as_wildcarded_query(state); }
-#line 2315 "queryparser/queryparser_internal.cc"
+#line 2334 "queryparser/queryparser_internal.cc"
         break;
       case 33:
-#line 1635 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1654 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 { yygotominor.yy13 = yymsp[0].minor.yy0->as_partial_query(state); }
-#line 2320 "queryparser/queryparser_internal.cc"
+#line 2339 "queryparser/queryparser_internal.cc"
         break;
       case 34:
-#line 1638 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1657 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 { yygotominor.yy13 = yymsp[-1].minor.yy1->as_phrase_query();   yy_destructor(19,&yymsp[-2].minor);
   yy_destructor(19,&yymsp[0].minor);
 }
-#line 2327 "queryparser/queryparser_internal.cc"
+#line 2346 "queryparser/queryparser_internal.cc"
         break;
       case 35:
-#line 1641 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1660 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 { yygotominor.yy13 = yymsp[0].minor.yy1->as_phrase_query(); }
-#line 2332 "queryparser/queryparser_internal.cc"
+#line 2351 "queryparser/queryparser_internal.cc"
         break;
       case 36:
-#line 1643 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1662 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy13 = yymsp[0].minor.yy60->as_group(state);
 }
-#line 2339 "queryparser/queryparser_internal.cc"
+#line 2358 "queryparser/queryparser_internal.cc"
         break;
       case 37:
-#line 1648 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1667 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 { yygotominor.yy13 = yymsp[0].minor.yy1->as_near_query(); }
-#line 2344 "queryparser/queryparser_internal.cc"
+#line 2363 "queryparser/queryparser_internal.cc"
         break;
       case 38:
-#line 1651 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1670 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 { yygotominor.yy13 = yymsp[0].minor.yy1->as_adj_query(); }
-#line 2349 "queryparser/queryparser_internal.cc"
+#line 2368 "queryparser/queryparser_internal.cc"
         break;
       case 39:
-#line 1654 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1673 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 { yygotominor.yy13 = yymsp[-1].minor.yy13;   yy_destructor(20,&yymsp[-2].minor);
   yy_destructor(21,&yymsp[0].minor);
 }
-#line 2356 "queryparser/queryparser_internal.cc"
+#line 2375 "queryparser/queryparser_internal.cc"
         break;
       case 40:
-#line 1656 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1675 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy13 = new Query(yymsp[0].minor.yy0->get_query_with_synonyms());
     delete yymsp[0].minor.yy0;
   yy_destructor(10,&yymsp[-1].minor);
 }
-#line 2365 "queryparser/queryparser_internal.cc"
+#line 2384 "queryparser/queryparser_internal.cc"
         break;
       case 41:
-#line 1667 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1686 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy1 = new TermList;
     yygotominor.yy1->add_positional_term(yymsp[0].minor.yy0);
 }
-#line 2373 "queryparser/queryparser_internal.cc"
+#line 2392 "queryparser/queryparser_internal.cc"
         break;
       case 42:
       case 44:
-#line 1672 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1691 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy1 = yymsp[-1].minor.yy1;
     yygotominor.yy1->add_positional_term(yymsp[0].minor.yy0);
 }
-#line 2382 "queryparser/queryparser_internal.cc"
+#line 2401 "queryparser/queryparser_internal.cc"
         break;
       case 43:
-#line 1684 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1703 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy1 = new TermList;
     yygotominor.yy1->add_positional_term(yymsp[-1].minor.yy0);
     yygotominor.yy1->add_positional_term(yymsp[0].minor.yy0);
 }
-#line 2391 "queryparser/queryparser_internal.cc"
+#line 2410 "queryparser/queryparser_internal.cc"
         break;
       case 45:
-#line 1701 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1720 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy60 = new TermGroup;
     yygotominor.yy60->add_term(yymsp[-1].minor.yy0);
     yygotominor.yy60->add_term(yymsp[0].minor.yy0);
 }
-#line 2400 "queryparser/queryparser_internal.cc"
+#line 2419 "queryparser/queryparser_internal.cc"
         break;
       case 46:
-#line 1707 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1726 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy60 = yymsp[-1].minor.yy60;
     yygotominor.yy60->add_term(yymsp[0].minor.yy0);
 }
-#line 2408 "queryparser/queryparser_internal.cc"
+#line 2427 "queryparser/queryparser_internal.cc"
         break;
       case 47:
       case 49:
-#line 1718 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1737 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy1 = new TermList;
     yygotominor.yy1->add_positional_term(yymsp[-2].minor.yy0);
@@ -2417,11 +2436,11 @@ static void yy_reduce(
 	delete yymsp[-1].minor.yy0;
     }
 }
-#line 2422 "queryparser/queryparser_internal.cc"
+#line 2441 "queryparser/queryparser_internal.cc"
         break;
       case 48:
       case 50:
-#line 1728 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1747 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 {
     yygotominor.yy1 = yymsp[-2].minor.yy1;
     yygotominor.yy1->add_positional_term(yymsp[0].minor.yy0);
@@ -2430,7 +2449,7 @@ static void yy_reduce(
 	delete yymsp[-1].minor.yy0;
     }
 }
-#line 2435 "queryparser/queryparser_internal.cc"
+#line 2454 "queryparser/queryparser_internal.cc"
         break;
   }
   yygoto = yyRuleInfo[yyruleno].lhs;
@@ -2455,11 +2474,11 @@ static void yy_parse_failed(
   while( !yypParser->yystack.empty() ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will be executed whenever the
   ** parser fails */
-#line 1304 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.5/xapian/xapian-core/queryparser/queryparser.lemony"
+#line 1323 "/data/home/olly/tmp/xapian-svn-snapshot/tags/1.0.7/xapian/xapian-core/queryparser/queryparser.lemony"
 
     // If we've not already set an error message, set a default one.
     if (!state->error) state->error = "parse error";
-#line 2465 "queryparser/queryparser_internal.cc"
+#line 2484 "queryparser/queryparser_internal.cc"
   ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
 
