@@ -35,14 +35,19 @@ using namespace std;
 
 class State;
 
+/** Information about how to handle a prefix in the query string. */
 struct PrefixInfo {
-    typedef enum {
-	FREE_TEXT,
-	BOOL_FILTER
-    } prefix_type;
-    PrefixInfo::prefix_type type;
-    string str;
-    PrefixInfo(PrefixInfo::prefix_type t, const string &s) : type(t), str(s) { }
+    /// Is this a boolean filter prefix?
+    bool filter;
+
+    /// Prefix strings.
+    list<string> prefixes;
+
+    PrefixInfo(bool filter_, const string & prefix)
+	: filter(filter_)
+    {
+	prefixes.push_back(prefix);
+    }
 };
 
 namespace Xapian {
@@ -62,13 +67,15 @@ class QueryParser::Internal : public Xapian::Internal::RefCntBase {
     multimap<string, string> unstem;
 
     // Map "from" -> "A" ; "subject" -> "C" ; "newsgroups" -> "G" ;
-    // "foobar" -> "XFOO"
-    map<string, PrefixInfo> prefixes;
+    // "foobar" -> "XFOO". FIXME: it does more than this now!
+    map<string, PrefixInfo> prefixmap;
 
     list<ValueRangeProcessor *> valrangeprocs;
 
     string corrected_query;
 
+    void add_prefix(const string &field, const string &prefix, bool filter);
+
     std::string parse_term(Utf8Iterator &it, const Utf8Iterator &end,
 			   bool &was_acronym);
 
