@@ -3,7 +3,7 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
  * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
- * Copyright 2006 Richard Boulton
+ * Copyright 2006,2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,6 +30,7 @@
 #include <string>
 
 #include <xapian.h>
+#include "backendmanager_local.h"
 #include "testsuite.h"
 #include "testutils.h"
 #include "utils.h"
@@ -491,6 +492,56 @@ DEFINE_TESTCASE(topercent1, backend) {
     return true;
 }
 
+// tests the percentage values returned
+DEFINE_TESTCASE(topercent2, backend) {
+    BackendManagerLocal local_manager;
+    local_manager.set_datadir(test_driver::get_srcdir() + "/testdata/");
+    Xapian::Enquire localenq(local_manager.get_database("apitest_simpledata"));
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+
+    int pct;
+
+    // First, test a search in which the top document scores 100%.
+    enquire.set_query(query("this"));
+    localenq.set_query(query("this"));
+    Xapian::MSet mymset = enquire.get_mset(0, 20);
+    Xapian::MSet localmset = localenq.get_mset(0, 20);
+
+    Xapian::MSetIterator i = mymset.begin();
+    TEST(i != mymset.end());
+    pct = mymset.convert_to_percent(i);
+    TEST_EQUAL(pct, 100);
+
+    TEST_EQUAL(mymset, localmset);
+    TEST(mset_range_is_same_percents(mymset, 0, localmset, 0, mymset.size()));
+
+    // A search in which the top document doesn't have 100%
+    Xapian::Query q = query(Xapian::Query::OP_OR,
+			    "this", "line", "paragraph", "rubbish");
+    enquire.set_query(q);
+    localenq.set_query(q);
+    mymset = enquire.get_mset(0, 20);
+    localmset = localenq.get_mset(0, 20);
+
+    i = mymset.begin();
+    TEST(i != mymset.end());
+    pct = mymset.convert_to_percent(i);
+    TEST_GREATER(pct, 65);
+    TEST_LESSER(pct, 75);
+
+    ++i;
+
+    TEST(i != mymset.end());
+    pct = mymset.convert_to_percent(i);
+    TEST_GREATER(pct, 40);
+    TEST_LESSER(pct, 50);
+
+    TEST_EQUAL(mymset, localmset);
+    TEST(mset_range_is_same_percents(mymset, 0, localmset, 0, mymset.size()));
+
+    return true;
+}
+
 class myExpandFunctor : public Xapian::ExpandDecider {
     public:
 	bool operator()(const string & tname) const {
@@ -619,6 +670,35 @@ DEFINE_TESTCASE(pctcutoff1, backend) {
     return true;
 }
 
+// Tests the percent cutoff option combined with collapsing
+DEFINE_TESTCASE(pctcutoff2, backend) {
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    enquire.set_query(Xapian::Query("this"));
+    enquire.set_query(Xapian::Query(Xapian::Query::OP_AND_NOT, Xapian::Query("this"), Xapian::Query("banana")));
+    Xapian::MSet mset = enquire.get_mset(0, 100);
+
+    if (verbose) {
+	tout << "Original mset pcts:";
+	print_mset_percentages(mset);
+	tout << "\n";
+    }
+
+    TEST(mset.size() >= 2);
+    TEST(mset[0].get_percent() - mset[1].get_percent() >= 2);
+
+    Xapian::percent cutoff = mset[0].get_percent() + mset[1].get_percent();
+    cutoff /= 2;
+
+    enquire.set_cutoff(cutoff);
+    enquire.set_collapse_key(1234); // Value which is always empty.
+
+    mset = enquire.get_mset(0, 1);
+    TEST_EQUAL(mset.size(), 1);
+    TEST_EQUAL(mset.get_matches_lower_bound(), 1);
+
+    return true;
+}
+
 // tests the cutoff option
 DEFINE_TESTCASE(cutoff1, backend) {
     Xapian::Enquire enquire(get_database("apitest_simpledata"));
@@ -1037,8 +1117,8 @@ DEFINE_TESTCASE(rsetmultidb1, backend && !multi) {
     mset_expect_order(mymset2a, 1, 2);
     mset_expect_order(mymset2b, 2, 1);
 
-    mset_range_is_same_weights(mymset1a, 0, mymset2a, 0, 2);
-    mset_range_is_same_weights(mymset1b, 0, mymset2b, 0, 2);
+    TEST(mset_range_is_same_weights(mymset1a, 0, mymset2a, 0, 2));
+    TEST(mset_range_is_same_weights(mymset1b, 0, mymset2b, 0, 2));
     TEST_NOT_EQUAL(mymset1a, mymset1b);
     TEST_NOT_EQUAL(mymset2a, mymset2b);
 
@@ -1800,6 +1880,190 @@ DEFINE_TESTCASE(valuerange1, backend) {
     return true;
 }
 
+// Regression test for Query::OP_VALUE_LE - used to return document IDs for
+// non-existent documents.
+DEFINE_TESTCASE(valuerange2, backend && writable) {
+    Xapian::WritableDatabase db = get_writable_database();
+    Xapian::Document doc;
+    doc.set_data("5");
+    doc.add_value(0, "5");
+    db.replace_document(5, doc);
+    Xapian::Enquire enq(db);
+
+    Xapian::Query query(Xapian::Query::OP_VALUE_LE, 0, "6");
+    enq.set_query(query);
+    Xapian::MSet mset = enq.get_mset(0, 20);
+
+    TEST_EQUAL(mset.size(), 1);
+    TEST_EQUAL(*(mset[0]), 5);
+    return true;
+}
+
+// Test for alldocs postlist with a sparse database.
+DEFINE_TESTCASE(alldocspl1, backend && writable) {
+    Xapian::WritableDatabase db = get_writable_database();
+    Xapian::Document doc;
+    doc.set_data("5");
+    doc.add_value(0, "5");
+    db.replace_document(5, doc);
+
+    Xapian::PostingIterator i = db.postlist_begin("");
+    TEST(i != db.postlist_end(""));
+    TEST_EQUAL(*i, 5);
+    TEST_EQUAL(i.get_doclength(), 0);
+    TEST_EQUAL(i.get_wdf(), 1);
+    ++i;
+    TEST(i == db.postlist_end(""));
+
+    return true;
+}
+
+// Test reading and writing a modified alldocspostlist.
+DEFINE_TESTCASE(alldocspl2, backend && writable) {
+    Xapian::PostingIterator i, end;
+    {
+	Xapian::WritableDatabase db = get_writable_database();
+	Xapian::Document doc;
+	doc.set_data("5");
+	doc.add_value(0, "5");
+	db.replace_document(5, doc);
+
+	// Test iterating before flushing the changes.
+	i = db.postlist_begin("");
+	end = db.postlist_end("");
+	TEST(i != end);
+	TEST_EQUAL(*i, 5);
+	TEST_EQUAL(i.get_doclength(), 0);
+	TEST_EQUAL(i.get_wdf(), 1);
+	++i;
+	TEST(i == end);
+
+	db.flush();
+
+	// Test iterating after flushing the changes.
+	i = db.postlist_begin("");
+	end = db.postlist_end("");
+	TEST(i != end);
+	TEST_EQUAL(*i, 5);
+	TEST_EQUAL(i.get_doclength(), 0);
+	TEST_EQUAL(i.get_wdf(), 1);
+	++i;
+	TEST(i == end);
+
+	// Add another document.
+	doc = Xapian::Document();
+	doc.set_data("5");
+	doc.add_value(0, "7");
+	db.replace_document(7, doc);
+
+	// Test iterating through before flushing the changes.
+	i = db.postlist_begin("");
+	end = db.postlist_end("");
+	TEST(i != end);
+	TEST_EQUAL(*i, 5);
+	TEST_EQUAL(i.get_doclength(), 0);
+	TEST_EQUAL(i.get_wdf(), 1);
+	++i;
+	TEST(i != end);
+	TEST_EQUAL(*i, 7);
+	TEST_EQUAL(i.get_doclength(), 0);
+	TEST_EQUAL(i.get_wdf(), 1);
+	++i;
+	TEST(i == end);
+
+	// Delete the first document.
+	db.delete_document(5);
+
+	// Test iterating through before flushing the changes.
+	i = db.postlist_begin("");
+	end = db.postlist_end("");
+	TEST(i != end);
+	TEST_EQUAL(*i, 7);
+	TEST_EQUAL(i.get_doclength(), 0);
+	TEST_EQUAL(i.get_wdf(), 1);
+	++i;
+	TEST(i == end);
+
+	// Test iterating through after flushing the changes, and dropping the reference to the main DB.
+	db.flush();
+	i = db.postlist_begin("");
+	end = db.postlist_end("");
+    }
+
+    TEST(i != end);
+    TEST_EQUAL(*i, 7);
+    TEST_EQUAL(i.get_doclength(), 0);
+    TEST_EQUAL(i.get_wdf(), 1);
+    ++i;
+    TEST(i == end);
+
+    return true;
+}
+
+// Feature test for Query::OP_VALUE_GE.
+DEFINE_TESTCASE(valuege1, backend) {
+    Xapian::Database db(get_database("apitest_phrase"));
+    Xapian::Enquire enq(db);
+    static const char * vals[] = {
+	"", " ", "a", "aa", "abcd", "e", "g", "h", "hzz", "i", "l", "z", NULL
+    };
+    for (const char **start = vals; *start; ++start) {
+	Xapian::Query query(Xapian::Query::OP_VALUE_GE, 1, *start);
+	enq.set_query(query);
+	Xapian::MSet mset = enq.get_mset(0, 20);
+	// Check that documents in the MSet match the value range filter.
+	set<Xapian::docid> matched;
+	Xapian::MSetIterator i;
+	for (i = mset.begin(); i != mset.end(); ++i) {
+	    matched.insert(*i);
+	    string value = db.get_document(*i).get_value(1);
+	    tout << "'" << *start << "' <= '" << value << "'" << endl;
+	    TEST(value >= *start);
+	}
+	// Check that documents not in the MSet don't match the value range filter.
+	for (Xapian::docid j = db.get_lastdocid(); j != 0; --j) {
+	    if (matched.find(j) == matched.end()) {
+		string value = db.get_document(j).get_value(1);
+		tout << value << " < '" << *start << "'" << endl;
+		TEST(value < *start);
+	    }
+	}
+    }
+    return true;
+}
+
+// Feature test for Query::OP_VALUE_LE.
+DEFINE_TESTCASE(valuele1, backend) {
+    Xapian::Database db(get_database("apitest_phrase"));
+    Xapian::Enquire enq(db);
+    static const char * vals[] = {
+	"", " ", "a", "aa", "abcd", "e", "g", "h", "hzz", "i", "l", "z", NULL
+    };
+    for (const char **end = vals; *end; ++end) {
+	Xapian::Query query(Xapian::Query::OP_VALUE_LE, 1, *end);
+	enq.set_query(query);
+	Xapian::MSet mset = enq.get_mset(0, 20);
+	// Check that documents in the MSet match the value range filter.
+	set<Xapian::docid> matched;
+	Xapian::MSetIterator i;
+	for (i = mset.begin(); i != mset.end(); ++i) {
+	    matched.insert(*i);
+	    string value = db.get_document(*i).get_value(1);
+	    tout << "'" << *end << "' <= '" << value << "'" << endl;
+	    TEST(value <= *end);
+	}
+	// Check that documents not in the MSet don't match the value range filter.
+	for (Xapian::docid j = db.get_lastdocid(); j != 0; --j) {
+	    if (matched.find(j) == matched.end()) {
+		string value = db.get_document(j).get_value(1);
+		tout << value << " < '" << *end << "'" << endl;
+		TEST(value > *end);
+	    }
+	}
+    }
+    return true;
+}
+
 // Feature test for Query::OP_SCALE_WEIGHT.
 DEFINE_TESTCASE(scaleweight1, backend) {
     Xapian::Database db(get_database("apitest_phrase"));
