@@ -269,12 +269,41 @@ DEFINE_TESTCASE(matchfunctor1, backend && !remote) {
     Xapian::MSetIterator i = mymset.begin();
     TEST(i != mymset.end());
     TEST_EQUAL(mymset.size(), 3);
+    TEST_EQUAL(mymset.get_matches_lower_bound(), 3);
+    TEST_EQUAL(mymset.get_matches_upper_bound(), 3);
+    TEST_EQUAL(mymset.get_matches_estimated(), 3);
     for ( ; i != mymset.end(); ++i) {
 	const Xapian::Document doc(i.get_document());
 	TEST(myfunctor(doc));
 	docid_checked[*i] = true;
     }
 
+    // Check that there are some documents which aren't accepted by the match
+    // decider.
+    mymset = enquire.get_mset(0, 100);
+    TEST(mymset.size() > 3);
+
+    // Check that the bounds are appropriate even if we don't ask for any
+    // actual matches.
+    mymset = enquire.get_mset(0, 0, 0, &myfunctor);
+    TEST_EQUAL(mymset.size(), 0);
+    TEST_EQUAL(mymset.get_matches_lower_bound(), 0);
+    TEST_EQUAL(mymset.get_matches_upper_bound(), 6);
+    TEST(mymset.get_matches_estimated() > 0);
+    TEST(mymset.get_matches_estimated() <= 6);
+
+    // Check that the bounds are appropriate if we ask for only one hit.
+    // (Regression test - until SVN 10256, we didn't reduce the lower_bound
+    // appropriately, and returned 6 here.)
+    mymset = enquire.get_mset(0, 1, 0, &myfunctor);
+    TEST_EQUAL(mymset.size(), 1);
+    TEST(mymset.get_matches_lower_bound() >= 1);
+    TEST(mymset.get_matches_lower_bound() <= 3);
+    TEST(mymset.get_matches_upper_bound() >= 3);
+    TEST(mymset.get_matches_upper_bound() <= 6);
+    TEST(mymset.get_matches_estimated() > 0);
+    TEST(mymset.get_matches_estimated() <= 6);
+
     // Check that the other documents don't satisfy the condition.
     for (Xapian::docid did = 1; did < docid_checked.size(); ++did) {
 	if (!docid_checked[did]) {
@@ -282,6 +311,42 @@ DEFINE_TESTCASE(matchfunctor1, backend && !remote) {
 	}
     }
 
+    // Check that the bounds are appropriate if a collapse key is used.
+    // Use a value which is never set so we don't actually discard anything.
+    enquire.set_collapse_key(99);
+    mymset = enquire.get_mset(0, 1, 0, &myfunctor);
+    TEST_EQUAL(mymset.size(), 1);
+    TEST(mymset.get_matches_lower_bound() >= 1);
+    TEST(mymset.get_matches_lower_bound() <= 3);
+    TEST(mymset.get_matches_upper_bound() >= 3);
+    TEST(mymset.get_matches_upper_bound() <= 6);
+    TEST(mymset.get_matches_estimated() > 0);
+    TEST(mymset.get_matches_estimated() <= 6);
+
+    // Check that the bounds are appropriate if a percentage cutoff is in
+    // use.  Set a 1% threshold so we don't actually discard anything.
+    enquire.set_collapse_key(Xapian::BAD_VALUENO);
+    enquire.set_cutoff(1);
+    mymset = enquire.get_mset(0, 1, 0, &myfunctor);
+    TEST_EQUAL(mymset.size(), 1);
+    TEST(mymset.get_matches_lower_bound() >= 1);
+    TEST(mymset.get_matches_lower_bound() <= 3);
+    TEST(mymset.get_matches_upper_bound() >= 3);
+    TEST(mymset.get_matches_upper_bound() <= 6);
+    TEST(mymset.get_matches_estimated() > 0);
+    TEST(mymset.get_matches_estimated() <= 6);
+
+    // And now with both a collapse key and percentage cutoff.
+    enquire.set_collapse_key(99);
+    mymset = enquire.get_mset(0, 1, 0, &myfunctor);
+    TEST_EQUAL(mymset.size(), 1);
+    TEST(mymset.get_matches_lower_bound() >= 1);
+    TEST(mymset.get_matches_lower_bound() <= 3);
+    TEST(mymset.get_matches_upper_bound() >= 3);
+    TEST(mymset.get_matches_upper_bound() <= 6);
+    TEST(mymset.get_matches_estimated() > 0);
+    TEST(mymset.get_matches_estimated() <= 6);
+
     return true;
 }
 
@@ -318,6 +383,49 @@ DEFINE_TESTCASE(matchfunctor2, backend && !remote) {
     return true;
 }
 
+class myMatchDecider2 : public Xapian::MatchDecider {
+    public:
+	bool operator()(const Xapian::Document &doc) const {
+	    // Note that this is not recommended usage of get_data()
+	    return doc.get_data().find("We produce") == string::npos;
+	}
+};
+
+
+// Regression test for lower bound using functor, sorting and collapsing.
+DEFINE_TESTCASE(matchfunctor3, backend) {
+    Xapian::Database db(get_database("etext"));
+    Xapian::Enquire enquire(db);
+    enquire.set_query(Xapian::Query(""));
+    enquire.set_collapse_key(12);
+    enquire.set_sort_by_value(11);
+
+    myMatchDecider2 myfunctor;
+
+    Xapian::MSet mymset1 = enquire.get_mset(0, 2, 0, NULL, &myfunctor);
+    Xapian::MSet mymset2 = enquire.get_mset(0, 1000, 0, NULL, &myfunctor);
+
+    // mymset2 should contain all the hits, so the statistics should be exact.
+    TEST_EQUAL(mymset2.get_matches_estimated(),
+	       mymset2.size());
+    TEST_EQUAL(mymset2.get_matches_lower_bound(),
+	       mymset2.get_matches_estimated());
+    TEST_EQUAL(mymset2.get_matches_estimated(),
+	       mymset2.get_matches_upper_bound());
+
+    // Check that the lower bound in mymset1 is not greater than the known
+    // number of hits.  This failed until revision 10811.
+    TEST_LESSER_OR_EQUAL(mymset1.get_matches_lower_bound(),
+			 mymset2.size());
+
+    // Check that the bounds for mymset1 make sense
+    TEST_LESSER_OR_EQUAL(mymset1.get_matches_lower_bound(), mymset1.get_matches_estimated());
+    TEST_LESSER_OR_EQUAL(mymset1.size(), mymset1.get_matches_upper_bound());
+    TEST_LESSER_OR_EQUAL(mymset1.get_matches_estimated(), mymset1.get_matches_upper_bound());
+
+    return true;
+}
+
 // tests that mset iterators on msets compare correctly.
 DEFINE_TESTCASE(msetiterator1, backend) {
     Xapian::Enquire enquire(get_database("apitest_simpledata"));
@@ -902,8 +1010,8 @@ DEFINE_TESTCASE(rsetmultidb2, backend && !multi) {
     mset_expect_order(mymset2a, 2, 5);
     mset_expect_order(mymset2b, 2, 5);
 
-    mset_range_is_same_weights(mymset1a, 0, mymset2a, 0, 2);
-    mset_range_is_same_weights(mymset1b, 0, mymset2b, 0, 2);
+    TEST(mset_range_is_same_weights(mymset1a, 0, mymset2a, 0, 2));
+    TEST(mset_range_is_same_weights(mymset1b, 0, mymset2b, 0, 2));
     TEST_NOT_EQUAL(mymset1a, mymset1b);
     TEST_NOT_EQUAL(mymset2a, mymset2b);
 
