@@ -251,11 +251,13 @@ DEFINE_TESTCASE(singlesubq1, !backend) {
 }
 
 DEFINE_TESTCASE(stemlangs1, !backend) {
-    unsigned lang_count = 0;
     string langs = Xapian::Stem::get_available_languages();
     tout << "available languages '" << langs << "'" << endl;
     TEST(!langs.empty());
 
+    // Also test the language codes.
+    langs += " da nl en fi fr de hu it no pt ro ru es sv tr";
+
     string::size_type i = 0;
     while (true) {
 	string::size_type spc = langs.find(' ', i);
@@ -263,8 +265,6 @@ DEFINE_TESTCASE(stemlangs1, !backend) {
 	// of language names.
 	TEST_NOT_EQUAL(i, spc);
 
-	++lang_count;
-
 	// Try making a stemmer for this language.  We should be able to create
 	// it without an exception being thrown.
 	string language = langs.substr(i, spc - i);
@@ -275,12 +275,12 @@ DEFINE_TESTCASE(stemlangs1, !backend) {
 	i = spc + 1;
     }
 
-    // Check that we actually had some languages to test.
-    TEST(lang_count > 0);
-
     // Check that we get an exception for a bogus language name.
     TEST_EXCEPTION(Xapian::InvalidArgumentError, Xapian::Stem stemmer("bogus"));
 
+    // Stem("") should give an object which doesn't change any input.
+    Xapian::Stem stem_nothing = Xapian::Stem("");
+
     return true;
 }
 
@@ -388,3 +388,10 @@ DEFINE_TESTCASE(scaleweight4, !backend) {
 
     return true;
 }
+
+// Check that Query(OP_VALUE_GE, 0, "") -> Query::MatchAll.
+DEFINE_TESTCASE(opvaluege1, !backend) {
+    Xapian::Query query(Xapian::Query::OP_VALUE_GE, 0, "");
+    TEST_STRINGS_EQUAL(query.get_description(), Xapian::Query::MatchAll.get_description());
+    return true;
+}
