@@ -1,7 +1,7 @@
 /** @file api_unicode.cc
- * @brief test the Unicode and Utf8 classes and functions.
+ * @brief test the Unicode and UTF-8 classes and functions.
  */
-/* Copyright (C) 2006,2007 Olly Betts
+/* Copyright (C) 2006,2007,2008 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,8 @@
 #include "apitest.h"
 #include "testutils.h"
 
+#include <cctype>
+
 using namespace std;
 
 struct testcase {
@@ -40,7 +42,7 @@ static const testcase testcases[] = {
     { 0, 0 }
 };
 
-// Test handling of invalid utf-8 is as desired.
+// Test handling of invalid UTF-8 is as desired.
 DEFINE_TESTCASE(utf8iterator1,!backend) {
     const testcase * p;
     for (p = testcases; p->a; ++p) {
@@ -79,7 +81,7 @@ static const testcase2 testcases2[] = {
     { 0, 0 }
 };
 
-// Test decoding of utf-8.
+// Test decoding of UTF-8.
 DEFINE_TESTCASE(utf8iterator2,!backend) {
     const testcase2 * p;
     for (p = testcases2; p->a; ++p) {
@@ -99,7 +101,7 @@ DEFINE_TESTCASE(unicode1,!backend) {
     TEST_EQUAL(Unicode::get_category('0'), Unicode::DECIMAL_DIGIT_NUMBER);
     TEST_EQUAL(Unicode::get_category('$'), Unicode::CURRENCY_SYMBOL);
     TEST_EQUAL(Unicode::get_category(0xa3), Unicode::CURRENCY_SYMBOL);
-    // 0x242 was added in Unicode 5.0.0.
+    // U+0242 was added in Unicode 5.0.0.
     TEST_EQUAL(Unicode::get_category(0x242), Unicode::LOWERCASE_LETTER);
     TEST_EQUAL(Unicode::get_category(0xFFFF), Unicode::UNASSIGNED);
     // Test characters outside BMP.
@@ -107,7 +109,50 @@ DEFINE_TESTCASE(unicode1,!backend) {
     TEST_EQUAL(Unicode::get_category(0x10FFFD), Unicode::PRIVATE_USE);
     TEST_EQUAL(Unicode::get_category(0x10FFFF), Unicode::UNASSIGNED);
     // Test some invalid Unicode values.
-    TEST_EQUAL(Unicode::get_category(0x11000), Unicode::UNASSIGNED);
+    TEST_EQUAL(Unicode::get_category(0x110000), Unicode::UNASSIGNED);
     TEST_EQUAL(Unicode::get_category(0xFFFFFFFF), Unicode::UNASSIGNED);
     return true;
 }
+
+DEFINE_TESTCASE(caseconvert1,!backend) {
+    using namespace Xapian;
+    for (unsigned ch = 0; ch < 128; ++ch) {
+	if (isupper((char)ch)) {
+	    TEST_EQUAL(Unicode::tolower(ch), unsigned(tolower((char)ch)));
+	} else {
+	    TEST_EQUAL(Unicode::tolower(ch), ch);
+	}
+	if (islower((char)ch)) {
+	    TEST_EQUAL(Unicode::toupper(ch), unsigned(toupper((char)ch)));
+	} else {
+	    TEST_EQUAL(Unicode::toupper(ch), ch);
+	}
+    }
+
+    // U+0242 was added in Unicode 5.0.0 as a lowercase form of U+0241.
+    TEST_EQUAL(Unicode::tolower(0x242), 0x242);
+    TEST_EQUAL(Unicode::toupper(0x242), 0x241);
+    TEST_EQUAL(Unicode::toupper(0x241), 0x241);
+    TEST_EQUAL(Unicode::tolower(0x241), 0x242);
+
+    // Pound currency symbol:
+    TEST_EQUAL(Unicode::tolower(0xa3), 0xa3);
+    TEST_EQUAL(Unicode::toupper(0xa3), 0xa3);
+    // Unassigned:
+    TEST_EQUAL(Unicode::tolower(0xFFFF), 0xFFFF);
+    TEST_EQUAL(Unicode::toupper(0xFFFF), 0xFFFF);
+    // Test characters outside BMP.
+    TEST_EQUAL(Unicode::tolower(0x10345), 0x10345);
+    TEST_EQUAL(Unicode::toupper(0x10345), 0x10345);
+    TEST_EQUAL(Unicode::tolower(0x10FFFD), 0x10FFFD);
+    TEST_EQUAL(Unicode::toupper(0x10FFFD), 0x10FFFD);
+    TEST_EQUAL(Unicode::tolower(0x10FFFF), 0x10FFFF);
+    TEST_EQUAL(Unicode::toupper(0x10FFFF), 0x10FFFF);
+    // Test some invalid Unicode values.
+    TEST_EQUAL(Unicode::tolower(0x110000), 0x110000);
+    TEST_EQUAL(Unicode::toupper(0x110000), 0x110000);
+    TEST_EQUAL(Unicode::tolower(0xFFFFFFFF), 0xFFFFFFFF);
+    TEST_EQUAL(Unicode::toupper(0xFFFFFFFF), 0xFFFFFFFF);
+
+    return true;
+}
