@@ -1875,6 +1875,25 @@ DEFINE_TESTCASE(metadata3, metadata) {
     return true;
 }
 
+// Regression test for adding a piece of metadata on its own before adding
+// other things.
+DEFINE_TESTCASE(metadata4, metadata) {
+    Xapian::WritableDatabase db = get_writable_database();
+
+    db.set_metadata("foo", "foo");
+    db.flush();
+
+    Xapian::Document doc;
+    doc.add_posting("foo", 1);
+    db.add_document(doc);
+
+    Xapian::Database dbr(get_writable_database_as_database());
+
+    return true;
+}
+
+
+
 // Test that adding a document with a really long term gives an error on
 // add_document() rather than on flush().
 DEFINE_TESTCASE(termtoolong1, writable) {
@@ -1896,7 +1915,16 @@ DEFINE_TESTCASE(termtoolong1, writable) {
 	Xapian::Document doc;
 	string term(i, 'X');
 	doc.add_term(term);
-	TEST_EXCEPTION(Xapian::InvalidArgumentError, db.add_document(doc));
+	try {
+	    db.add_document(doc);
+	    TEST_AND_EXPLAIN(false, "Expecting exception InvalidArgumentError");
+	} catch (const Xapian::InvalidArgumentError &e) {
+	    // Check that the max length is correctly expressed in the
+	    // exception message - we've got this wrong in two different ways
+	    // in the past!
+	    tout << e.get_msg() << endl;
+	    TEST(e.get_msg().find("Term too long (> 245)") != string::npos);
+	}
     }
 
     for (Xapian::doccount j = 240; j <= 245; ++j) {
@@ -1909,6 +1937,23 @@ DEFINE_TESTCASE(termtoolong1, writable) {
 
     db.flush();
 
+    {
+	// Currently flint doesn't allow
+	Xapian::Document doc;
+	doc.add_term(string(126, '\0'));
+	db.add_document(doc);
+	try {
+	    db.flush();
+	    TEST_AND_EXPLAIN(false, "Expecting exception InvalidArgumentError");
+	} catch (const Xapian::InvalidArgumentError &e) {
+	    // Check that the max length is correctly expressed in the
+	    // exception message - we've got this wrong in two different ways
+	    // in the past!
+	    tout << e.get_msg() << endl;
+	    TEST(e.get_msg().find(" is 252 bytes") != string::npos);
+	}
+    }
+
     return true;
 }
 
@@ -1954,3 +1999,16 @@ DEFINE_TESTCASE(postlist7, writable) {
 
     return true;
 }
+
+/// Regression test of reading after writing but not flushing.
+DEFINE_TESTCASE(writeread1, writable && metadata) {
+    Xapian::WritableDatabase db_w = get_writable_database();
+    db_w.set_metadata("1", "2");
+    string longitem(20000, 'j');
+    db_w.set_metadata("2", longitem);
+
+    string readitem = db_w.get_metadata("2");
+    TEST_EQUAL(readitem, longitem);
+
+    return true;
+}
